@@ -18,41 +18,19 @@
 
 #include "BasicImageView.h.moc"
 #include "ImageTransformation.h"
+#include "ImagePresentation.h"
 #include "Dpm.h"
 #include "Dpi.h"
 
 BasicImageView::BasicImageView(QImage const& image)
-:	ImageViewBase(
-		image, QImage(),
-		ImageTransformation(image.rect(), Dpm(image))
-	)
+:	ImageViewBase(image, QImage(), ImagePresentation(QTransform(), QRectF(image.rect()))),
+	m_dragHandler(*this),
+	m_zoomHandler(*this)
 {
+	rootInteractionHandler().makeLastFollower(m_dragHandler);
+	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 BasicImageView::~BasicImageView()
 {
 }
-
-void
-BasicImageView::wheelEvent(QWheelEvent* const event)
-{
-	handleZooming(event);
-}
-
-void
-BasicImageView::mousePressEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-BasicImageView::mouseReleaseEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-BasicImageView::mouseMoveEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
