@@ -1,6 +1,6 @@
 /*
 	Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+	Copyright (C)  Joseph Artsimovich <joseph_a@mail.ru>
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -21,9 +21,7 @@
 
 #include "AutoRemovingFile.h"
 #include <QStackedWidget>
-#include <QWidget>
 #include <boost/intrusive/list.hpp>
-#include <boost/function.hpp>
 
 class QImage;
 
@@ -33,11 +31,8 @@ class DebugImageView :
 		boost::intrusive::link_mode<boost::intrusive::auto_unlink>
 	>
 {
-    Q_OBJECT
 public:
-	DebugImageView(AutoRemovingFile file,
-		boost::function<QWidget* (QImage const&)> const& image_view_factory =
-		boost::function<QWidget* (QImage const&)>(), QWidget* parent = 0);
+	DebugImageView(AutoRemovingFile file, QWidget* parent = 0);
 
 	/**
 	 * Tells this widget to either display the actual image or just
@@ -51,7 +46,6 @@ private:
 	void imageLoaded(QImage const& image);
 
 	AutoRemovingFile m_file;
-	boost::function<QWidget* (QImage const&)> m_imageViewFactory;
 	QWidget* m_pPlaceholderWidget;
 	bool m_isLive;
 };
