@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "FixDpiDialog.h"
-#include "FixDpiDialog.moc"
+#include "FixDpiDialog.h.moc"
 #include <QAbstractItemModel>
 #include <QSortFilterProxyModel>
 #include <QModelIndex>
@@ -30,14 +29,12 @@
 #include <QColor>
 #include <Qt>
 #include <QDebug>
-#ifndef Q_MOC_RUN
+#include <boost/foreach.hpp>
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
-#endif
 #include <vector>
 #include <algorithm>
 #include <assert.h>
-#include "settings/ini_keys.h"
 
 // To be able to use it in QVariant
 Q_DECLARE_METATYPE(ImageMetadata)
@@ -236,22 +233,10 @@ FixDpiDialog::FixDpiDialog(std::vector<ImageFileInfo> const& files, QWidget* par
 	m_errorPalette = m_normalPalette;
 	m_errorPalette.setColor(QPalette::Text, Qt::red);
 	
-    QStringList sl = QSettings().value(_key_dpi_predefined_list, _key_dpi_predefined_list_def).toString().split(',',QString::KeepEmptyParts);
-    foreach (QString s, sl) {
-        QStringList nums = s.split('x');
-        bool ok = nums.count() >= 2;
-        int x = 0; int y = 0;
-        if (ok) {
-            x = nums[0].trimmed().toInt(&ok);
-            if (ok) {
-                y = nums[1].trimmed().toInt(&ok);
-                if (ok) {
-                    dpiCombo->addItem(QString("%1 x %2").arg(x).arg(y), QSize(x, y));
-                }
-            }
-        }
-    }
-
+	dpiCombo->addItem("300 x 300", QSize(300, 300));
+	dpiCombo->addItem("400 x 400", QSize(400, 400));
+	dpiCombo->addItem("600 x 600", QSize(600, 600));
+	
 	tabWidget->setTabText(NEED_FIXING_TAB, tr("Need Fixing"));
 	tabWidget->setTabText(ALL_PAGES_TAB, tr("All Pages"));
 	undefinedDpiView->setModel(m_ptrUndefinedDpiPages->model()),
@@ -353,12 +338,11 @@ FixDpiDialog::dpiValueChanged()
 		return;
 	}
 	
-	
 	if (metadata.isDpiOK()) {
 		applyBtn->setEnabled(true);
 		return;
 	}
-		
+	
 	applyBtn->setEnabled(false);
 }
 
@@ -487,9 +471,6 @@ FixDpiDialog::decorateDpiInputField(QLineEdit* field, ImageMetadata::DpiStatus d
 		case ImageMetadata::DPI_UNDEFINED:
 			field->setToolTip(QString());
 			break;
-		case ImageMetadata::DPI_TOO_LARGE:
-			field->setToolTip(tr("DPI is too large and most likely wrong."));
-			break;
 		case ImageMetadata::DPI_TOO_SMALL:
 			field->setToolTip(tr("DPI is too small. Even if it's correct, you are not going to get acceptable results with it."));
 			break;
