@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovigh@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,19 +18,11 @@
 
 #include "ImageLoader.h"
 #include "TiffReader.h"
-#include "ImageId.h"
-#include <QImageReader>
 #include <QImage>
 #include <QString>
 #include <QIODevice>
 #include <QFile>
 
-QImage
-ImageLoader::load(ImageId const& image_id)
-{
-	return load(image_id.filePath(), image_id.zeroBasedPage());
-}
-
 QImage
 ImageLoader::load(QString const& file_path, int const page_num)
 {
@@ -38,13 +30,7 @@ ImageLoader::load(QString const& file_path, int const page_num)
 	if (!file.open(QIODevice::ReadOnly)) {
 		return QImage();
 	}
-
-    if (file_path.startsWith(":")) {
-        // internally empty pages are represented as multipage image although they're just links to the same single page image in app resources
-        return load(file, 0);
-    }
-
-    return load(file, page_num);
+	return load(file, page_num);
 }
 
 QImage
@@ -60,6 +46,6 @@ ImageLoader::load(QIODevice& io_dev, int const page_num)
 	}
 	
 	QImage image;
-    QImageReader(&io_dev).read(&image);
+	image.load(&io_dev, 0);
 	return image;
 }
