@@ -18,14 +18,20 @@
 
 #include "ImageViewBase.h.moc"
 #include "NonCopyable.h"
+#include "ImagePresentation.h"
+#include "OpenGLSupport.h"
 #include "PixmapRenderer.h"
 #include "BackgroundExecutor.h"
 #include "Dpm.h"
 #include "Dpi.h"
+#include "ScopedIncDec.h"
 #include "imageproc/PolygonUtils.h"
 #include "imageproc/Transform.h"
+#include "config.h"
+#include <QScrollBar>
 #include <QPointer>
 #include <QAtomicInt>
+#include <QPaintEngine>
 #include <QPainter>
 #include <QPainterPath>
 #include <QBrush>
@@ -37,12 +43,19 @@
 #include <QResizeEvent>
 #include <QStatusTipEvent>
 #include <QApplication>
+#include <QSettings>
+#include <QVariant>
 #include <Qt>
 #include <QDebug>
 #include <algorithm>
 #include <assert.h>
 #include <math.h>
 
+#ifdef ENABLE_OPENGL
+#include <QGLWidget>
+#include <QGLFormat>
+#endif
+
 using namespace imageproc;
 
 class ImageViewBase::HqTransformTask :
@@ -117,20 +130,56 @@ private:
 };
 
 
+class ImageViewBase::TransformChangeWatcher
+{
+public:
+	TransformChangeWatcher(ImageViewBase& owner);
+
+	~TransformChangeWatcher();
+private:
+	ImageViewBase& m_rOwner;
+	QTransform m_imageToVirtual;
+	QTransform m_virtualToWidget;
+	QRectF m_virtualDisplayArea;
+};
+
+
 ImageViewBase::ImageViewBase(
 	QImage const& image, QImage const& downscaled_image,
-	ImageTransformation const& pre_transform,
-	Margins const& margins)
-:	m_defaultStatusTip(tr("Use the mouse wheel to zoom.  When zoomed, dragging is possible.")),
-	m_unrestrictedDragStatusTip(tr("Unrestricted dragging is possible by holding down the Shift key.")),
-	m_image(image),
-	m_imageToVirt(pre_transform),
+	ImagePresentation const& presentation, Margins const& margins)
+:	m_image(image),
+	m_virtualImageCropArea(presentation.cropArea()),
+	m_virtualDisplayArea(presentation.displayArea()),
+	m_imageToVirtual(presentation.transform()),
+	m_virtualToImage(presentation.transform().inverted()),
+	m_lastMaximumViewportSize(maximumViewportSize()),
 	m_margins(margins),
 	m_zoom(1.0),
-	m_currentCursorShape(Qt::ArrowCursor),
-	m_isDraggingInProgress(false),
+	m_transformChangeWatchersActive(0),
+	m_ignoreScrollEvents(0),
 	m_hqTransformEnabled(true)
 {
+#ifdef ENABLE_OPENGL
+	if (QSettings().value("settings/use_3d_acceleration", false) != false) {
+		if (OpenGLSupport::supported()) {
+			QGLFormat format;
+			format.setSampleBuffers(true);
+			format.setStencil(true);
+			format.setAlpha(true);
+			format.setRgba(true);
+			format.setDepth(false);
+
+			// Most of hardware refuses to work for us with direct rendering enabled.
+			format.setDirectRendering(false);
+
+			setViewport(new QGLWidget(format));
+		}
+	}
+#endif
+
+	setFrameShape(QFrame::NoFrame);
+	viewport()->setFocusPolicy(Qt::WheelFocus);
+
 	if (downscaled_image.isNull()) {
 		m_pixmap = QPixmap::fromImage(createDownscaledImage(image));
 	} else {
@@ -143,11 +192,7 @@ ImageViewBase::ImageViewBase(
 	);
 	
 	m_widgetFocalPoint = centeredWidgetFocalPoint();
-	m_pixmapFocalPoint = m_imageToVirt.transformBack().map(
-		m_imageToVirt.resultingRect().center()
-	);
-	
-	ensureStatusTip(defaultStatusTip());
+	m_pixmapFocalPoint = m_virtualToImage.map(virtualDisplayRect().center());
 	
 	m_timer.setSingleShot(true);
 	m_timer.setInterval(150); // msec
@@ -157,6 +202,16 @@ ImageViewBase::ImageViewBase(
 	);
 	
 	updateWidgetTransformAndFixFocalPoint(CENTER_IF_FITS);
+
+	interactionState().setDefaultStatusTip(
+		tr("Use the mouse wheel to zoom.  When zoomed, dragging is possible.")
+	);
+	ensureStatusTip(interactionState().statusTip());
+
+	connect(horizontalScrollBar(), SIGNAL(sliderReleased()), SLOT(updateScrollBars()));
+	connect(verticalScrollBar(), SIGNAL(sliderReleased()), SLOT(updateScrollBars()));
+	connect(horizontalScrollBar(), SIGNAL(valueChanged(int)), SLOT(reactToScrollBars()));
+	connect(verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(reactToScrollBars()));
 }
 
 ImageViewBase::~ImageViewBase()
@@ -211,285 +266,103 @@ ImageViewBase::createDownscaledImage(QImage const& image)
 	return transform(image, xform, QRect(0, 0, d_w, d_h), Qt::white);
 }
 
-void
-ImageViewBase::paintEvent(QPaintEvent* const event)
-{
-	QPainter painter(this);
-	painter.save();
-	
-#if !defined(Q_WS_X11)
-	double const xscale = m_virtualToWidget.m11();
-	
-	// Width of a source pixel in mm, as it's displayed on screen.
-	double const pixel_width = widthMM() * xscale / width();
-	
-	// On X11 SmoothPixmapTransform is too slow.
-	painter.setRenderHint(QPainter::SmoothPixmapTransform, pixel_width < 0.5);
-#endif
-	
-	validateHqPixmap();
-	if (!m_hqPixmap.isNull()) {
-		painter.drawPixmap(m_hqPixmapPos, m_hqPixmap);
-	} else {
-		painter.setWorldTransform(
-			m_pixmapToImage * m_imageToVirt.transform()
-			* m_virtualToWidget
-		);
-		PixmapRenderer::drawPixmap(painter, m_pixmap);
-		
-		// If m_ptrHqTransformTask is not null after validateHqPixmap(),
-		// that means the right version is being built.
-		if (!m_ptrHqTransformTask && m_hqTransformEnabled) {
-			// Schedule construction of a high quality version.
-			m_timer.start();
-		}
-	}
-	
-	painter.setRenderHints(QPainter::Antialiasing, true);
-	painter.setWorldMatrixEnabled(false);
-	
-	// Cover parts of the image that should not be visible with background.
-	// Note that because of Qt::WA_OpaquePaintEvent attribute, we need
-	// to paint the whole widget, which we do here.
-	
-	QPolygonF const image_area(
-		PolygonUtils::round(
-			m_virtualToWidget.map(
-				m_imageToVirt.transform().map(
-					m_imageToVirt.origRect()
-				)
-			)
-		)
-	);
-	QPolygonF const crop_area(
-		PolygonUtils::round(
-			m_virtualToWidget.map(m_imageToVirt.resultingCropArea())
-		)
-	);
-	
-	QPolygonF const intersected_area(
-		PolygonUtils::round(image_area.intersected(crop_area))
-	);
-	
-	QPainterPath intersected_path;
-	intersected_path.addPolygon(intersected_area);
-	
-	QPainterPath containing_path;
-	containing_path.addRect(rect());
-	
-	QBrush const brush(palette().color(QPalette::Window));
-	QPen pen(brush, 1.0);
-	pen.setCosmetic(true);
-	
-	// By using a pen with the same color as the brush, we essentially
-	// expanding the area we are going to draw.  It's necessary because
-	// XRender doesn't provide subpixel accuracy.
-	
-	painter.setPen(pen);
-	painter.setBrush(brush);
-	painter.drawPath(containing_path.subtracted(intersected_path));
-	
-	painter.restore();
-	
-	painter.setWorldTransform(m_virtualToWidget);
-	paintOverImage(painter);
-}
-
-void
-ImageViewBase::paintOverImage(QPainter& painter)
-{
-}
-
-void
-ImageViewBase::resizeEvent(QResizeEvent* event)
-{
-	if (event->oldSize().isEmpty()) {
-		m_widgetFocalPoint = centeredWidgetFocalPoint();
-	} else {
-		double const x_fraction = m_widgetFocalPoint.x()
-				/ event->oldSize().width();
-		double const y_fraction = m_widgetFocalPoint.y()
-				/ event->oldSize().height();
-		m_widgetFocalPoint.setX(x_fraction * event->size().width());
-		m_widgetFocalPoint.setY(y_fraction * event->size().height());
-	}
-	
-	updateWidgetTransform();
-}
-
-void
-ImageViewBase::hideEvent(QHideEvent*)
-{
-	m_isDraggingInProgress = false;
-}
-
-void
-ImageViewBase::handleZooming(QWheelEvent* const event, ZoomFocus const zoom_focus)
-{
-	if (m_isDraggingInProgress) {
-		return;
-	}
-	
-	double const degrees = event->delta() / 8.0;
-	m_zoom *= pow(2.0, degrees / 60.0); // 2 times zoom for every 60 degrees
-	if (m_zoom < 1.0) {
-		m_zoom = 1.0;
-	}
-	
-	QPointF focus_point;
-	switch (zoom_focus) {
-		case ZOOM_FOCUS_CENTER:
-			focus_point = QRectF(rect()).center();
-			break;
-		case ZOOM_FOCUS_CURSOR:
-			focus_point = event->pos() + QPointF(0.5, 0.5);
-			break;
-	}
-	setWidgetFocalPointWithoutMoving(focus_point);
-	
-	updateWidgetTransform();
-	update();
-}
-
-void
-ImageViewBase::handleImageDragging(QMouseEvent* const event)
-{
-	switch (event->type()) {
-		case QEvent::MouseButtonPress:
-			if (!m_isDraggingInProgress && event->button() == Qt::LeftButton) {
-				m_lastMousePos = event->pos();
-				m_isDraggingInProgress = true;
-				ensureStatusTip(m_unrestrictedDragStatusTip);
-			}
-			break;
-		case QEvent::MouseButtonRelease:
-			if (m_isDraggingInProgress && event->button() == Qt::LeftButton) {
-				m_isDraggingInProgress= false;
-				ensureStatusTip(defaultStatusTip());
-			}
-			break;
-		case QEvent::MouseMove:
-			if (m_isDraggingInProgress) {
-				QPoint movement(event->pos());
-				movement -= m_lastMousePos;
-				m_lastMousePos = event->pos();
-				
-				QPointF adjusted_fp(m_widgetFocalPoint);
-				adjusted_fp += movement;
-				
-				if (event->modifiers() & Qt::ShiftModifier) {
-					setNewWidgetFP(adjusted_fp);
-				} else {
-					adjustAndSetNewWidgetFP(adjusted_fp);
-				}
-				
-				update();
-			}
-		default:;
-	}
-}
-
-bool
-ImageViewBase::isDraggingPossible() const
-{
-	QRectF const image_rect(m_imageToVirt.resultingRect());
-	QRectF const widget_rect(m_virtualToWidget.mapRect(image_rect));
-	if (widget_rect.top() <= -1.0) {
-		return true;
-	}
-	if (widget_rect.left() <= -1.0) {
-		return true;
-	}
-	if (widget_rect.bottom() >= height() + 1) {
-		return true;
-	}
-	if (widget_rect.right() >= width() + 1) {
-		return true;
-	}
-	return false;
-}
-
 QRectF
-ImageViewBase::marginsRect() const
+ImageViewBase::viewportRect() const
 {
-	QRectF r(rect());
+	QRectF const viewport_rect(QPointF(0, 0), maximumViewportSize());
+	QRectF r(viewport_rect);
 	r.adjust(
 		m_margins.left(), m_margins.top(),
 		-m_margins.right(), -m_margins.bottom()
 	);
 	if (r.isEmpty()) {
-		return QRectF(r.center(), r.center());
+		return QRectF(viewport_rect.center(), viewport_rect.center());
 	}
 	return r;
 }
 
 QRectF
-ImageViewBase::getVisibleWidgetRect() const
+ImageViewBase::getOccupiedWidgetRect() const
 {
-	QRectF const image_rect(m_imageToVirt.resultingRect());
-	QRectF const widget_rect(m_virtualToWidget.mapRect(image_rect));
-	return widget_rect.intersected(marginsRect());
+	QRectF const widget_rect(m_virtualToWidget.mapRect(virtualDisplayRect()));
+	return widget_rect.intersected(viewportRect());
 }
 
 void
 ImageViewBase::setWidgetFocalPoint(QPointF const& widget_fp)
 {
-	if (widget_fp != m_widgetFocalPoint) {
-		m_widgetFocalPoint = widget_fp;
+	setNewWidgetFP(widget_fp, /*update =*/true);
+}
+
+void
+ImageViewBase::adjustAndSetWidgetFocalPoint(QPointF const& widget_fp)
+{
+	adjustAndSetNewWidgetFP(widget_fp, /*update=*/true);
+}
+
+void
+ImageViewBase::zoom(double zoom)
+{
+	if (m_zoom != zoom) {
+		m_zoom = zoom;
 		updateWidgetTransform();
 		update();
 	}
 }
 
 void
-ImageViewBase::resetZoom()
-{
-	if (m_zoom != 1.0) {
-		m_zoom = 1.0;
-		updateWidgetTransform();
-		update();
-	}
-}
-
-void
-ImageViewBase::updateTransform(ImageTransformation const& image_to_virt)
+ImageViewBase::updateTransform(ImagePresentation const& presentation)
 {
+	TransformChangeWatcher const watcher(*this);
 	TempFocalPointAdjuster const temp_fp(*this);
-	
-	m_imageToVirt = image_to_virt;
+
+	m_imageToVirtual = presentation.transform();
+	m_virtualToImage = m_imageToVirtual.inverted();
+	m_virtualImageCropArea = presentation.cropArea();
+	m_virtualDisplayArea = presentation.displayArea();
+
 	updateWidgetTransform();
 	update();
 }
 
 void
 ImageViewBase::updateTransformAndFixFocalPoint(
-	ImageTransformation const& image_to_virt, FocalPointMode const mode)
+	ImagePresentation const& presentation, FocalPointMode const mode)
 {
+	TransformChangeWatcher const watcher(*this);
 	TempFocalPointAdjuster const temp_fp(*this);
 	
-	m_imageToVirt = image_to_virt;
+	m_imageToVirtual = presentation.transform();
+	m_virtualToImage = m_imageToVirtual.inverted();
+	m_virtualImageCropArea = presentation.cropArea();
+	m_virtualDisplayArea = presentation.displayArea();
+
 	updateWidgetTransformAndFixFocalPoint(mode);
 	update();
 }
 
 void
-ImageViewBase::updateTransformPreservingScale(
-	ImageTransformation const& image_to_virt)
+ImageViewBase::updateTransformPreservingScale(ImagePresentation const& presentation)
 {
+	TransformChangeWatcher const watcher(*this);
 	TempFocalPointAdjuster const temp_fp(*this);
 	
 	// An arbitrary line in image coordinates.
 	QLineF const image_line(0.0, 0.0, 1.0, 1.0);
 	
 	QLineF const widget_line_before(
-		m_virtualToWidget.map(m_imageToVirt.transform().map(image_line))
+		(m_imageToVirtual * m_virtualToWidget).map(image_line)
 	);
 	
-	m_imageToVirt = image_to_virt;
+	m_imageToVirtual = presentation.transform();
+	m_virtualToImage = m_imageToVirtual.inverted();
+	m_virtualImageCropArea = presentation.cropArea();
+	m_virtualDisplayArea = presentation.displayArea();
+
 	updateWidgetTransform();
 	
 	QLineF const widget_line_after(
-		m_virtualToWidget.map(m_imageToVirt.transform().map(image_line))
+		(m_imageToVirtual * m_virtualToWidget).map(image_line)
 	);
 	
 	m_zoom *= widget_line_before.length() / widget_line_after.length();
@@ -498,15 +371,6 @@ ImageViewBase::updateTransformPreservingScale(
 	update();
 }
 
-void
-ImageViewBase::ensureCursorShape(Qt::CursorShape const cursor_shape)
-{
-	if (cursor_shape != m_currentCursorShape) {
-		m_currentCursorShape = cursor_shape;
-		setCursor(cursor_shape);
-	}
-}
-
 void
 ImageViewBase::ensureStatusTip(QString const& status_tip)
 {
@@ -518,21 +382,314 @@ ImageViewBase::ensureStatusTip(QString const& status_tip)
 		return;
 	}
 	
-	setStatusTip(status_tip);
+	viewport()->setStatusTip(status_tip);
 	
-	if (underMouse()) {
+	if (viewport()->underMouse()) {
 		// Note that setStatusTip() alone is not enough,
 		// as it's only taken into account when the mouse
 		// enters the widget.
-		QStatusTipEvent tip_event(status_tip);
-		QApplication::sendEvent(this, &tip_event);
+		// Also note that we use postEvent() rather than sendEvent(),
+		// because sendEvent() may immediately process other events.
+		QApplication::postEvent(viewport(), new QStatusTipEvent(status_tip));
 	}
 }
 
-QString
-ImageViewBase::defaultStatusTip() const
+void
+ImageViewBase::paintEvent(QPaintEvent* event)
 {
-	return m_defaultStatusTip;
+	QPainter painter(viewport());
+	painter.save();
+
+	// On X11 (except with OpenGL), SmoothPixmapTransform is too slow, so don't enable it.
+	bool smooth_pixmap_ok = true;
+#if defined(Q_WS_X11)
+	smooth_pixmap_ok = viewport()->inherits("QGLWidget");
+#endif
+	if (smooth_pixmap_ok) {
+		double const xscale = m_virtualToWidget.m11();
+
+		// Width of a source pixel in mm, as it's displayed on screen.
+		double const pixel_width = widthMM() * xscale / width();
+
+		// Disable antialiasing for large zoom levels.
+		painter.setRenderHint(QPainter::SmoothPixmapTransform, pixel_width < 0.5);
+	}
+
+	if (validateHqPixmap()) {
+		// HQ pixmap maps one to one to screen pixels, so antialiasing is not necessary.
+		painter.setRenderHint(QPainter::SmoothPixmapTransform, false);
+		painter.drawPixmap(m_hqPixmapPos, m_hqPixmap);
+	} else {
+		scheduleHqVersionRebuild();
+
+		painter.setWorldTransform(
+			m_pixmapToImage * m_imageToVirtual * m_virtualToWidget
+		);
+		PixmapRenderer::drawPixmap(painter, m_pixmap);
+	}
+
+	painter.setRenderHints(QPainter::Antialiasing, true);
+	painter.setWorldMatrixEnabled(false);
+
+	// Cover parts of the image that should not be visible with background.
+	// Note that because of Qt::WA_OpaquePaintEvent attribute, we need
+	// to paint the whole widget, which we do here.
+
+	QPolygonF const image_area(
+		PolygonUtils::round(
+			m_virtualToWidget.map(
+				m_imageToVirtual.map(QRectF(m_image.rect()))
+			)
+		)
+	);
+	QPolygonF const crop_area(
+		PolygonUtils::round(m_virtualToWidget.map(m_virtualImageCropArea))
+	);
+
+	QPolygonF const intersected_area(
+		PolygonUtils::round(image_area.intersected(crop_area))
+	);
+
+	QPainterPath intersected_path;
+	intersected_path.addPolygon(intersected_area);
+
+	QPainterPath containing_path;
+	containing_path.addRect(viewport()->rect());
+
+	QBrush const brush(palette().color(QPalette::Window));
+	QPen pen(brush, 1.0);
+	pen.setCosmetic(true);
+
+	// By using a pen with the same color as the brush, we essentially
+	// expanding the area we are going to draw.  It's necessary because
+	// XRender doesn't provide subpixel accuracy.
+
+	painter.setPen(pen);
+	painter.setBrush(brush);
+	painter.drawPath(containing_path.subtracted(intersected_path));
+
+	painter.restore();
+
+	painter.setWorldTransform(m_virtualToWidget);
+
+	m_interactionState.resetProximity();
+	if (!m_interactionState.captured()) {
+		m_rootInteractionHandler.proximityUpdate(
+			QPointF(0.5, 0.5) + mapFromGlobal(QCursor::pos()), m_interactionState
+		);
+		updateStatusTipAndCursor();
+	}
+
+	m_rootInteractionHandler.paint(painter, m_interactionState);
+}
+
+void
+ImageViewBase::keyPressEvent(QKeyEvent* event)
+{
+	event->setAccepted(false);
+	m_rootInteractionHandler.keyPressEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::keyReleaseEvent(QKeyEvent* event)
+{
+	event->setAccepted(false);
+	m_rootInteractionHandler.keyReleaseEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::mousePressEvent(QMouseEvent* event)
+{
+	m_interactionState.resetProximity();
+	if (!m_interactionState.captured()) {
+		m_rootInteractionHandler.proximityUpdate(
+			QPointF(0.5, 0.5) + event->pos(), m_interactionState
+		);
+	}
+
+	event->setAccepted(false);
+	m_rootInteractionHandler.mousePressEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::mouseReleaseEvent(QMouseEvent* event)
+{
+	m_interactionState.resetProximity();
+	if (!m_interactionState.captured()) {
+		m_rootInteractionHandler.proximityUpdate(
+			QPointF(0.5, 0.5) + event->pos(), m_interactionState
+		);
+	}
+
+	event->setAccepted(false);
+	m_rootInteractionHandler.mouseReleaseEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::mouseMoveEvent(QMouseEvent* event)
+{
+	m_interactionState.resetProximity();
+	if (!m_interactionState.captured()) {
+		m_rootInteractionHandler.proximityUpdate(
+			QPointF(0.5, 0.5) + event->pos(), m_interactionState
+		);
+	}
+
+	event->setAccepted(false);
+	m_rootInteractionHandler.mouseMoveEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::wheelEvent(QWheelEvent* event)
+{
+	event->setAccepted(false);
+	m_rootInteractionHandler.wheelEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::contextMenuEvent(QContextMenuEvent* event)
+{
+	event->setAccepted(false);
+	m_rootInteractionHandler.contextMenuEvent(event, m_interactionState);
+	event->setAccepted(true);
+	updateStatusTipAndCursor();
+}
+
+void
+ImageViewBase::resizeEvent(QResizeEvent* event)
+{
+	ScopedIncDec<int> const guard(m_ignoreScrollEvents);
+
+	if (maximumViewportSize() != m_lastMaximumViewportSize) {
+		m_lastMaximumViewportSize = maximumViewportSize();
+		m_widgetFocalPoint = centeredWidgetFocalPoint();
+		updateWidgetTransform();
+	} else {
+		TempFocalPointAdjuster const temp_fp(*this, QPointF(0, 0));
+		updateTransformPreservingScale(
+			ImagePresentation(m_imageToVirtual, m_virtualImageCropArea, m_virtualDisplayArea)
+		);
+	}
+}
+
+/**
+ * Called when any of the transformations change.
+ */
+void
+ImageViewBase::transformChanged()
+{
+	updateScrollBars();
+}
+
+void
+ImageViewBase::updateScrollBars()
+{
+	if (verticalScrollBar()->isSliderDown() || horizontalScrollBar()->isSliderDown()) {
+		return;
+	}
+
+	TransformChangeWatcher const watcher(*this);
+	ScopedIncDec<int> const guard(m_ignoreScrollEvents);
+
+	QRectF const viewport(viewportRect());
+	QRectF const picture(m_virtualToWidget.mapRect(virtualDisplayRect()));
+
+	QPointF const viewport_center(viewport.center());
+	QPointF const picture_center(picture.center());
+
+	double const xval = picture_center.x();
+	double xmin, xmax; // Minimum and maximum positions for picture center.
+	if (picture_center.x() < viewport_center.x()) {
+		xmin = std::min<double>(xval, viewport.right() - 0.5 * picture.width());
+		xmax = std::max<double>(viewport_center.x(), viewport.left() + 0.5 * picture.width());
+	} else {
+		xmax = std::max<double>(xval, viewport.left() + 0.5 * picture.width());
+		xmin = std::min<double>(viewport_center.x(), viewport.right() - 0.5 * picture.width());
+	}
+
+	double const yval = picture_center.y();
+	double ymin, ymax; // Minimum and maximum positions for picture center.
+	if (picture_center.y() < viewport_center.y()) {
+		ymin = std::min<double>(yval, viewport.bottom() - 0.5 * picture.height());
+		ymax = std::max<double>(viewport_center.y(), viewport.top() + 0.5 * picture.height());
+	} else {
+		ymax = std::max<double>(yval, viewport.top() + 0.5 * picture.height());
+		ymin = std::min<double>(viewport_center.y(), viewport.bottom() - 0.5 * picture.height());
+	}
+
+	int const xrange = (int)ceil(xmax - xmin);
+	int const yrange = (int)ceil(ymax - ymin);
+	int const xfirst = 0;
+	int const xlast = xrange - 1;
+	int const yfirst = 0;
+	int const ylast = yrange - 1;
+
+	// We are going to map scrollbar coordinates to widget coordinates
+	// of the central point of the display area using a linear function.
+	// f(x) = ax + b
+
+	// xmin = xa * xlast + xb
+	// xmax = xa * xfirst + xb
+	double const xa = (xfirst == xlast) ? 1 : (xmax - xmin) / (xfirst - xlast);
+	double const xb = xmax - xa * xfirst;
+	double const ya = (yfirst == ylast) ? 1 : (ymax - ymin) / (yfirst - ylast);
+	double const yb = ymax - ya * yfirst;
+
+	// Inverse transformation.
+	// xlast = ixa * xmin + ixb
+	// xfirst = ixa * xmax + ixb
+	double const ixa = (xmax == xmin) ? 1 : (xfirst - xlast) / (xmax - xmin);
+	double const ixb = xfirst - ixa * xmax;
+	double const iya = (ymax == ymin) ? 1 : (yfirst - ylast) / (ymax - ymin);
+	double const iyb = yfirst - iya * ymax;
+
+	m_scrollTransform.setMatrix(xa, 0, 0, 0, ya, 0, xb, yb, 1);
+
+	int const xcur = qRound(ixa * xval + ixb);
+	int const ycur = qRound(iya * yval + iyb);
+
+	horizontalScrollBar()->setRange(xfirst, xlast);
+	verticalScrollBar()->setRange(yfirst, ylast);
+
+	horizontalScrollBar()->setValue(xcur);
+	verticalScrollBar()->setValue(ycur);
+
+	horizontalScrollBar()->setPageStep(qRound(viewport.width()));
+	verticalScrollBar()->setPageStep(qRound(viewport.height()));
+}
+
+void
+ImageViewBase::reactToScrollBars()
+{
+	if (m_ignoreScrollEvents) {
+		return;
+	}
+
+	TransformChangeWatcher const watcher(*this);
+
+	QPointF const raw_position(
+		horizontalScrollBar()->value(), verticalScrollBar()->value()
+	);
+	QPointF const new_fp(m_scrollTransform.map(raw_position));
+	QPointF const old_fp(getWidgetFocalPoint());
+
+	m_pixmapFocalPoint = m_virtualToImage.map(m_virtualDisplayArea.center());
+	m_widgetFocalPoint = new_fp;
+	updateWidgetTransform();
+
+	setWidgetFocalPointWithoutMoving(old_fp);
 }
 
 /**
@@ -545,30 +702,24 @@ ImageViewBase::defaultStatusTip() const
 void
 ImageViewBase::updateWidgetTransform()
 {
-	QRectF const virt_rect(m_imageToVirt.resultingRect());
-	QPointF const virt_origin(
-		m_imageToVirt.transform().map(
-			m_pixmapFocalPoint
-		)
-	);
+	TransformChangeWatcher const watcher(*this);
+
+	QRectF const virt_rect(virtualDisplayRect());
+	QPointF const virt_origin(m_imageToVirtual.map(m_pixmapFocalPoint));
 	QPointF const widget_origin(m_widgetFocalPoint);
 	
 	QSizeF zoom1_widget_size(virt_rect.size());
-	zoom1_widget_size.scale(marginsRect().size(), Qt::KeepAspectRatio);
+	zoom1_widget_size.scale(viewportRect().size(), Qt::KeepAspectRatio);
 	
 	double const zoom1_x = zoom1_widget_size.width() / virt_rect.width();
 	double const zoom1_y = zoom1_widget_size.height() / virt_rect.height();
 	
-	QTransform t1;
-	t1.translate(-virt_origin.x(), -virt_origin.y());
-	
-	QTransform t2;
-	t2.scale(zoom1_x * m_zoom, zoom1_y * m_zoom);
-	
-	QTransform t3;
-	t3.translate(widget_origin.x(), widget_origin.y());
-	
-	m_virtualToWidget = t1 * t2 * t3;
+	QTransform xform;
+	xform.translate(-virt_origin.x(), -virt_origin.y());
+	xform *= QTransform().scale(zoom1_x * m_zoom, zoom1_y * m_zoom);
+	xform *= QTransform().translate(widget_origin.x(), widget_origin.y());
+
+	m_virtualToWidget = xform;
 	m_widgetToVirtual = m_virtualToWidget.inverted();
 }
 
@@ -581,6 +732,8 @@ ImageViewBase::updateWidgetTransform()
 void
 ImageViewBase::updateWidgetTransformAndFixFocalPoint(FocalPointMode const mode)
 {
+	TransformChangeWatcher const watcher(*this);
+
 	// This must go before getIdealWidgetFocalPoint(), as it
 	// recalculates m_virtualToWidget, that is used by
 	// getIdealWidgetFocalPoint().
@@ -615,12 +768,10 @@ QPointF
 ImageViewBase::getIdealWidgetFocalPoint(FocalPointMode const mode) const
 {
 	// Widget rect reduced by margins.
-	QRectF const display_area(marginsRect());
+	QRectF const display_area(viewportRect());
 	
 	// The virtual image rectangle in widget coordinates.
-	QRectF const image_area(
-		m_virtualToWidget.mapRect(m_imageToVirt.resultingRect())
-	);
+	QRectF const image_area(m_virtualToWidget.mapRect(virtualDisplayRect()));
 	
 	// Unused display space from each side.
 	double const left_margin = image_area.left() - display_area.left();
@@ -668,11 +819,14 @@ ImageViewBase::getIdealWidgetFocalPoint(FocalPointMode const mode) const
 }
 
 void
-ImageViewBase::setNewWidgetFP(QPointF const widget_fp)
+ImageViewBase::setNewWidgetFP(QPointF const widget_fp, bool const update)
 {
 	if (widget_fp != m_widgetFocalPoint) {
 		m_widgetFocalPoint = widget_fp;
 		updateWidgetTransform();
+		if (update) {
+			this->update();
+		}
 	}
 }
 
@@ -686,7 +840,8 @@ ImageViewBase::setNewWidgetFP(QPointF const widget_fp)
  * \param proposed_widget_fp The proposed value for m_widgetFocalPoint.
  */
 void
-ImageViewBase::adjustAndSetNewWidgetFP(QPointF const proposed_widget_fp)
+ImageViewBase::adjustAndSetNewWidgetFP(
+	QPointF const proposed_widget_fp, bool const update)
 {
 	// We first apply the proposed focal point, and only then
 	// calculate the ideal one.  That's done because
@@ -695,7 +850,7 @@ ImageViewBase::adjustAndSetNewWidgetFP(QPointF const proposed_widget_fp)
 	// We don't want the ideal focal point to be equal to the current
 	// one, as that would disallow any movements.
 	QPointF const old_widget_fp(m_widgetFocalPoint);
-	setNewWidgetFP(proposed_widget_fp);
+	setNewWidgetFP(proposed_widget_fp, update);
 	
 	QPointF const ideal_widget_fp(getIdealWidgetFocalPoint(CENTER_IF_FITS));
 	
@@ -726,6 +881,9 @@ ImageViewBase::adjustAndSetNewWidgetFP(QPointF const proposed_widget_fp)
 	if (adjusted_widget_fp != m_widgetFocalPoint) {
 		m_widgetFocalPoint = adjusted_widget_fp;
 		updateWidgetTransform();
+		if (update) {
+			this->update();
+		}
 	}
 }
 
@@ -735,32 +893,75 @@ ImageViewBase::adjustAndSetNewWidgetFP(QPointF const proposed_widget_fp)
 QPointF
 ImageViewBase::centeredWidgetFocalPoint() const
 {
-	return marginsRect().center();
+	return viewportRect().center();
 }
 
-/**
- * Sets m_widgetFocalPoint and recalculates m_pixmapFocalPoints
- * so that the image is not moved.
- */
 void
 ImageViewBase::setWidgetFocalPointWithoutMoving(QPointF const new_widget_fp)
 {
 	m_widgetFocalPoint = new_widget_fp;
-	m_pixmapFocalPoint = m_imageToVirt.transformBack().map(
+	m_pixmapFocalPoint = m_virtualToImage.map(
 		m_widgetToVirtual.map(m_widgetFocalPoint)
 	);
 }
 
+/**
+ * Returns true if m_hqPixmap is valid and up to date.
+ */
+bool
+ImageViewBase::validateHqPixmap() const
+{
+	if (!m_hqTransformEnabled) {
+		return false;
+	}
+
+	if (m_hqPixmap.isNull()) {
+		return false;
+	}
+
+	if (m_hqSourceId != m_image.cacheKey()) {
+		return false;
+	}
+
+	if (m_hqXform != m_imageToVirtual * m_virtualToWidget) {
+		return false;
+	}
+
+	return true;
+}
+
+void
+ImageViewBase::scheduleHqVersionRebuild()
+{
+	QTransform const xform(m_imageToVirtual * m_virtualToWidget);
+
+	if (!m_timer.isActive() || m_potentialHqXform != xform) {
+		if (m_ptrHqTransformTask.get()) {
+			m_ptrHqTransformTask->cancel();
+			m_ptrHqTransformTask.reset();
+		}
+		m_potentialHqXform = xform;
+	}
+	m_timer.start();
+}
+
 void
 ImageViewBase::initiateBuildingHqVersion()
 {
-	if (!m_hqTransformEnabled) {
+	if (validateHqPixmap()) {
 		return;
 	}
-	
-	QTransform const xform(m_imageToVirt.transform() * m_virtualToWidget);
+
+	m_hqPixmap = QPixmap();
+
+	if (m_ptrHqTransformTask.get()) {
+		m_ptrHqTransformTask->cancel();
+		m_ptrHqTransformTask.reset();
+	}
+
+	QTransform const xform(m_imageToVirtual * m_virtualToWidget);
 	IntrusivePtr<HqTransformTask> const task(
-		new HqTransformTask(this, m_image, xform, size())
+		new HqTransformTask(this, m_image, xform, viewport()->size())
 	);
 	
 	backgroundExecutor().enqueueTask(task);
@@ -787,25 +988,23 @@ ImageViewBase::hqVersionBuilt(
 	update();
 }
 
-/**
- * Resets m_hqPixmap and cancels the transformation task if
- * the high-quality pixmap was built (or is being built) with
- * a transformation other than the current one or from the
- * source image other than the current one, or if high-quality
- * transformations are currently disabled.
- */
 void
-ImageViewBase::validateHqPixmap()
+ImageViewBase::updateStatusTipAndCursor()
 {
-	if (!m_hqTransformEnabled ||
-			m_hqXform != m_imageToVirt.transform() * m_virtualToWidget
-			|| m_hqSourceId != m_image.cacheKey()) {
-		m_hqPixmap = QPixmap();
-		if (m_ptrHqTransformTask.get()) {
-			m_ptrHqTransformTask->cancel();
-			m_ptrHqTransformTask.reset();
-		}
-	}
+	updateStatusTip();
+	updateCursor();
+}
+
+void
+ImageViewBase::updateStatusTip()
+{
+	ensureStatusTip(m_interactionState.statusTip());
+}
+
+void
+ImageViewBase::updateCursor()
+{
+	viewport()->setCursor(m_interactionState.cursor());
 }
 
 BackgroundExecutor&
@@ -906,3 +1105,26 @@ ImageViewBase::TempFocalPointAdjuster::~TempFocalPointAdjuster()
 {
 	m_rObj.setWidgetFocalPointWithoutMoving(m_origWidgetFP);
 }
+
+
+/*================== ImageViewBase::TransformChangeWatcher ================*/
+
+ImageViewBase::TransformChangeWatcher::TransformChangeWatcher(ImageViewBase& owner)
+:	m_rOwner(owner),
+	m_imageToVirtual(owner.m_imageToVirtual),
+	m_virtualToWidget(owner.m_virtualToWidget),
+	m_virtualDisplayArea(owner.m_virtualDisplayArea)
+{
+	++m_rOwner.m_transformChangeWatchersActive;
+}
+
+ImageViewBase::TransformChangeWatcher::~TransformChangeWatcher()
+{
+	if (--m_rOwner.m_transformChangeWatchersActive == 0) {
+		if (m_imageToVirtual != m_rOwner.m_imageToVirtual ||
+				m_virtualToWidget != m_rOwner.m_virtualToWidget ||
+				m_virtualDisplayArea != m_rOwner.m_virtualDisplayArea) {
+			m_rOwner.transformChanged();
+		}
+	}
+}
