@@ -19,11 +19,13 @@
 #ifndef IMAGEVIEWBASE_H_
 #define IMAGEVIEWBASE_H_
 
-#include "ImageTransformation.h"
 #include "Margins.h"
 #include "IntrusivePtr.h"
+#include "InteractionHandler.h"
+#include "InteractionState.h"
 #include <QTimer>
 #include <QWidget>
+#include <QAbstractScrollArea>
 #include <QPixmap>
 #include <QImage>
 #include <QString>
@@ -36,6 +38,7 @@
 
 class QPainter;
 class BackgroundExecutor;
+class ImagePresentation;
 
 /**
  * \brief The base class for widgets that display and manipulate images.
@@ -53,10 +56,12 @@ class BackgroundExecutor;
  *
  * \see m_pixmapToImage, m_imageToVirt, m_virtualToWidget, m_widgetToVirtual.
  */
-class ImageViewBase : public QWidget
+class ImageViewBase : public QAbstractScrollArea
 {
 	Q_OBJECT
 public:
+	enum FocalPointMode { CENTER_IF_FITS, DONT_CENTER };
+
 	/**
 	 * \brief ImageViewBase constructor.
 	 *
@@ -68,16 +73,16 @@ public:
 	 *        to speed up real-time rendering of high-resolution
 	 *        images.  Note that the delayed high quality transform
 	 *        operates on the original image, not the downscaled one.
-	 * \param pre_transform A logical transformation applied on \p image.
+	 * \param image_presentation Specifies transformation from image
+	 *        pixel coordinates to virtual image coordinates, along
+	 *        with some other properties.
 	 * \param margins Reserve extra space near the widget borders.
 	 *        The units are widget pixels.  This reserved area may
-	 *        still be covered by parts of the image that are outside
-	 *        of pre_transform.resultingRect().
+	 *        be used for custom drawing or custom controls.
 	 */
 	ImageViewBase(
 		QImage const& image, QImage const& downscaled_image,
-		ImageTransformation const& pre_transform,
-		Margins const& margins = Margins());
+		ImagePresentation const& presentation, Margins const& margins = Margins());
 	
 	virtual ~ImageViewBase();
 	
@@ -99,73 +104,43 @@ public:
 	 * \return The image downscaled by an unspecified degree.
 	 */
 	static QImage createDownscaledImage(QImage const& image);
-protected:
-	enum ZoomFocus { ZOOM_FOCUS_CENTER, ZOOM_FOCUS_CURSOR };
-	
-	enum FocalPointMode { CENTER_IF_FITS, DONT_CENTER };
-	
-	/**
-	 * \brief Repaint the widget.
-	 *
-	 * \note Don't override this one.  Override paintOverImage() instead.
-	 */
-	virtual void paintEvent(QPaintEvent* event);
-	
-	virtual void paintOverImage(QPainter& painter);
-	
-	/**
-	 * \brief Handle widget resizing.
-	 *
-	 * \note If overriden, call this version first!
-	 */
-	virtual void resizeEvent(QResizeEvent* event);
-	
-	/**
-	 * \brief Cancels ongoing image dragging.
-	 *
-	 * \note If overriden, call this version first!
-	 */
-	virtual void hideEvent(QHideEvent* event);
-	
-	/**
-	 * \brief Performs image zooming.
-	 *
-	 * To be called from subclasses.
-	 */
-	void handleZooming(QWheelEvent* event, ZoomFocus = ZOOM_FOCUS_CURSOR);
-	
-	/**
-	 * \brief Possibly starts or stops image dragging.
-	 *
-	 * To be called from subclasses on the following events:
-	 * MouseButtonPress, MouseButtonRelease, MouseMove.
-	 */
-	void handleImageDragging(QMouseEvent* event);
-	
-	ImageTransformation const& imageToVirt() const { return m_imageToVirt; }
-	
+
+	InteractionHandler& rootInteractionHandler() { return m_rootInteractionHandler; }
+
+	InteractionState& interactionState() { return m_interactionState; }
+
+	InteractionState const& interactionState() const { return m_interactionState; }
+
+	QTransform const& imageToVirtual() const { return m_imageToVirtual; }
+
+	QTransform const& virtualToImage() const { return m_virtualToImage; }
+
 	QTransform const& virtualToWidget() const { return m_virtualToWidget; }
-	
+
 	QTransform const& widgetToVirtual() const { return m_widgetToVirtual; }
-	
-	bool isDraggingInProgress() const { return m_isDraggingInProgress; }
-	
-	/**
-	 * Returns true if any part of the image is off-screen.
-	 */
-	bool isDraggingPossible() const;
-	
-	/**
-	 * Returns the widget area reduced by margins.
-	 */
-	QRectF marginsRect() const;
-	
+
+	QTransform imageToWidget() const { return m_imageToVirtual * m_virtualToWidget; }
+
+	QTransform widgetToImage() const { return m_widgetToVirtual * m_virtualToImage; }
+
+	void update() { viewport()->update(); }
+
+	QRectF const& virtualDisplayRect() const { return m_virtualDisplayArea; }
+
 	/**
 	 * Get the bounding box of the image as it appears on the screen,
 	 * in widget coordinates.
 	 */
-	QRectF getVisibleWidgetRect() const;
-	
+	QRectF getOccupiedWidgetRect() const;
+
+	/**
+	 * \brief A better version of setStatusTip().
+	 *
+	 * Unlike setStatusTip(), this method will display the tooltip
+	 * immediately, not when the mouse enters the widget next time.
+	 */
+	void ensureStatusTip(QString const& status_tip);
+
 	/**
 	 * \brief Get the focal point in widget coordinates.
 	 *
@@ -177,95 +152,127 @@ protected:
 	 * As a result, the image will be moved by delta widget pixels.
 	 */
 	QPointF getWidgetFocalPoint() const { return m_widgetFocalPoint; }
-	
+
 	/**
 	 * \brief Set the focal point in widget coordinates.
 	 *
+	 * This one may be used for unrestricted dragging (with Shift button).
+	 *
 	 * \see getWidgetFocalPoint()
 	 */
 	void setWidgetFocalPoint(QPointF const& widget_fp);
-	
+
 	/**
-	 * \brief Resets the zoom to the default value.
+	 * \brief Set the focal point in widget coordinates, after adjustring
+	 *        it to avoid wasting of widget space.
 	 *
-	 * The default zoom is such that the image can fit into the widget.
+	 * This one may be used for resticted dragging (the default one in ST).
+	 *
+	 * \see getWidgetFocalPoint()
+	 * \see setWidgetFocalPoint()
 	 */
-	void resetZoom();
-	
+	void adjustAndSetWidgetFocalPoint(QPointF const& widget_fp);
+
+	/**
+	 * \brief Sets the widget focal point and recalculates the pixmap focal
+	 *        focal point so that the image is not moved on screen.
+	 */
+	void setWidgetFocalPointWithoutMoving(QPointF new_widget_fp);
+
 	/**
 	 * \brief Updates image-to-virtual and recalculates
 	 *        virtual-to-widget transformations.
 	 */
-	void updateTransform(ImageTransformation const& image_to_virt);
-	
+	void updateTransform(ImagePresentation const& presentation);
+
 	/**
 	 * \brief Same as updateTransform(), but adjusts the focal point
 	 *        to improve screen space usage.
 	 */
 	void updateTransformAndFixFocalPoint(
-		ImageTransformation const& image_to_virt, FocalPointMode mode);
-	
+		ImagePresentation const& presentation, FocalPointMode mode);
+
 	/**
 	 * \brief Same as updateTransform(), but preserves the visual image scale.
 	 */
-	void updateTransformPreservingScale(
-		ImageTransformation const& image_to_virt);
+	void updateTransformPreservingScale(ImagePresentation const& presentation);
+
+	/**
+	 * \brief Sets the zoom level.
+	 *
+	 * Zoom level 1.0 means such a zoom that makes the image fit the widget.
+	 * Zooming will take into account the current widget and pixmap focal
+	 * points.  To zoom to a specific point, for example the mouse position,
+	 * call setWidgetFocalPointWithoutMoving() first.
+	 */
+	void zoom(double zoom);
+
+	static BackgroundExecutor& backgroundExecutor();
+protected:
+	virtual void paintEvent(QPaintEvent* event);
+
+	virtual void keyPressEvent(QKeyEvent* event);
+
+	virtual void keyReleaseEvent(QKeyEvent* event);
+
+	virtual void mousePressEvent(QMouseEvent* event);
+
+	virtual void mouseReleaseEvent(QMouseEvent* event);
+
+	virtual void mouseMoveEvent(QMouseEvent* event);
+
+	virtual void wheelEvent(QWheelEvent* event);
+
+	virtual void contextMenuEvent(QContextMenuEvent* event);
+
+	virtual void resizeEvent(QResizeEvent* event);
 	
 	/**
-	 * \brief A faster version of setCursor().
-	 *
-	 * This method checks if the shape we want to set is already set,
-	 * and if so, does nothing.
-	 * \note Calls to this method must not be mixed with calls to setCursor()
-	 *       and unsetCursor().
+	 * Returns the maximum viewport size (as if scrollbars are hidden)
+	 * reduced by margins.
 	 */
-	void ensureCursorShape(Qt::CursorShape cursor_shape);
-	
-	/**
-	 * \brief A better version of setStatusTip().
-	 *
-	 * Unlike setStatusTip(), this method will display the tooltip
-	 * immediately, not when the mouse enters the widget next time.
-	 */
-	void ensureStatusTip(QString const& status_tip);
-	
-	/**
-	 * \brief Returns the status tip to be associated with this widget
-	 *        in its default state.
-	 *
-	 * Subclasses may reimplement this method.
-	 */
-	virtual QString defaultStatusTip() const;
+	QRectF viewportRect() const;
 private slots:
 	void initiateBuildingHqVersion();
+
+	void updateScrollBars();
+
+	void reactToScrollBars();
 private:
 	class HqTransformTask;
 	class TempFocalPointAdjuster;
-	
+	class TransformChangeWatcher;
+
+	void transformChanged();
+
 	void updateWidgetTransform();
 	
 	void updateWidgetTransformAndFixFocalPoint(FocalPointMode mode);
 	
 	QPointF getIdealWidgetFocalPoint(FocalPointMode mode) const;
 	
-	void setNewWidgetFP(QPointF widget_fp);
+	void setNewWidgetFP(QPointF widget_fp, bool update = false);
 	
-	void adjustAndSetNewWidgetFP(QPointF proposed_widget_fp);
+	void adjustAndSetNewWidgetFP(QPointF proposed_widget_fp, bool update = false);
 	
 	QPointF centeredWidgetFocalPoint() const;
 	
-	void setWidgetFocalPointWithoutMoving(QPointF new_widget_fp);
-	
+	bool validateHqPixmap() const;
+
+	void scheduleHqVersionRebuild();
+
 	void hqVersionBuilt(QPoint const& origin, QImage const& image);
+
+	void updateStatusTipAndCursor();
+
+	void updateStatusTip();
+
+	void updateCursor();
 	
-	void validateHqPixmap();
-	
-	static BackgroundExecutor& backgroundExecutor();
-	
-	QString m_defaultStatusTip;
-	
-	QString m_unrestrictedDragStatusTip;
-	
+	InteractionHandler m_rootInteractionHandler;
+
+	InteractionState m_interactionState;
+
 	/**
 	 * The client-side image.  Used to build a high-quality version
 	 * for delayed rendering.
@@ -299,11 +306,16 @@ private:
 	 * It's used to detect if m_hqPixmap needs to be rebuild.
 	 */
 	QTransform m_hqXform;
+
+	/**
+	 * Used to check if we need to extend the delay before building m_hqPixmap.
+	 */
+	QTransform m_potentialHqXform;
 	
 	/**
 	 * The ID (QImage::cacheKey()) of the image that was used
 	 * to build m_hqPixmap.  It's used to detect if m_hqPixmap
-	 * needs to be rebuild.
+	 * needs to be rebuilt.
 	 */
 	qint64 m_hqSourceId;
 	
@@ -311,18 +323,35 @@ private:
 	 * The pending (if any) high quality transformation task.
 	 */
 	IntrusivePtr<HqTransformTask> m_ptrHqTransformTask;
-	
+
 	/**
 	 * Transformation from m_pixmap coordinates to m_image coordinates.
 	 */
 	QTransform m_pixmapToImage;
 	
 	/**
-	 * A set of logical transformations on m_image, to translate
-	 * from image to virtual image coordinates and back.
+	 * The area of the virtual image to be displayed.
+	 * Everything outside of it will be cropped.
 	 */
-	ImageTransformation m_imageToVirt;
+	QPolygonF m_virtualImageCropArea;
+
+	/**
+	 * The area in virtual image coordinates to be displayed.
+	 * The idea is that it can be larger than m_virtualImageCropArea
+	 * to reserve space for custom drawing or controls.
+	 */
+	QRectF m_virtualDisplayArea;
+
+	/**
+	 * A transformation from original to virtual image coordinates.
+	 */
+	QTransform m_imageToVirtual;
 	
+	/**
+	 * A transformation from virtual to original image coordinates.
+	 */
+	QTransform m_virtualToImage;
+
 	/**
 	 * Transformation from virtual image coordinates to widget coordinates.
 	 */
@@ -332,6 +361,12 @@ private:
 	 * Transformation from widget coordinates to virtual image coordinates.
 	 */
 	QTransform m_widgetToVirtual;
+
+	/**
+	 * Transforms scroll bar values to corresponding positions of the display
+	 * area (its central point) in widget coordinates.
+	 */
+	QTransform m_scrollTransform;
 	
 	/**
 	 * An arbitrary point in widget coordinates that corresponds
@@ -349,12 +384,12 @@ private:
 	 * m_widgetFocalPoint for zooming into a specific position.
 	 */
 	QPointF m_pixmapFocalPoint;
-	
+
 	/**
-	 * Used for dragging the image.  Holds the last cursor position
-	 * (in widget coordinates) that was processed in this dragging session.
+	 * Used to distinguish between resizes induced by scrollbars (dis)appearing
+	 * and other factors.
 	 */
-	QPoint m_lastMousePos;
+	QSize m_lastMaximumViewportSize;
 	
 	/**
 	 * The number of pixels to be left blank at each side of the widget.
@@ -366,12 +401,9 @@ private:
 	 */
 	double m_zoom;
 	
-	/**
-	 * The current cursor shape, cached to improve performance.
-	 */
-	Qt::CursorShape m_currentCursorShape;
-	
-	bool m_isDraggingInProgress;
+	int m_transformChangeWatchersActive;
+
+	int m_ignoreScrollEvents;
 	
 	bool m_hqTransformEnabled;
 };
