@@ -31,7 +31,10 @@
 #include "Utils.h"
 #include "FilterOptionsWidget.h"
 #include "ErrorWidget.h"
+#include "AutoRemovingFile.h"
 #include "DebugImages.h"
+#include "DebugImageView.h"
+#include "TabbedDebugImages.h"
 #include "BasicImageView.h"
 #include "ProjectWriter.h"
 #include "ProjectReader.h"
@@ -45,6 +48,7 @@
 #include "ImageMetadataLoader.h"
 #include "OrthogonalRotation.h"
 #include "FixDpiSinglePageDialog.h"
+#include "SettingsDialog.h"
 #include "filters/fix_orientation/Filter.h"
 #include "filters/fix_orientation/Task.h"
 #include "filters/fix_orientation/CacheDrivenTask.h"
@@ -66,7 +70,7 @@
 #include "LoadFileTask.h"
 #include "CompositeCacheDrivenTask.h"
 #include "ScopedIncDec.h"
-#include "ui_RemoveFileDialog.h"
+#include "ui_RemovePagesDialog.h"
 #include <boost/foreach.hpp>
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
@@ -127,7 +131,7 @@ MainWindow::MainWindow()
 	
 	setupThumbView(); // Expects m_ptrThumbSequence to be initialized.
 	
-	m_ptrTabbedDebugImages.reset(new QTabWidget);
+	m_ptrTabbedDebugImages.reset(new TabbedDebugImages);
 	
 	m_debug = actionDebug->isChecked();
 	m_pImageFrameLayout = new QStackedLayout(imageViewFrame);
@@ -187,6 +191,11 @@ MainWindow::MainWindow()
 		actionDebug, SIGNAL(toggled(bool)),
 		this, SLOT(debugToggled(bool))
 	);
+
+	connect(
+		actionSettings, SIGNAL(triggered(bool)),
+		this, SLOT(openSettingsDialog())
+	);
 	
 	connect(
 		actionNewProject, SIGNAL(triggered(bool)),
@@ -622,15 +631,16 @@ MainWindow::setImageWidget(
 		m_imageWidgetCleanup.add(widget);
 	}
 	
-	if (!debug_images || debug_images->items().empty()) {
+	if (!debug_images || debug_images->empty()) {
 		m_pImageFrameLayout->addWidget(widget);
 	} else {
 		m_ptrTabbedDebugImages->addTab(widget, "Main");
-		BOOST_FOREACH (DebugImages::Item& item, debug_images->items()) {
-			QWidget* widget = new BasicImageView(item.image());
+		AutoRemovingFile file;
+		QString label;
+		while (!(file = debug_images->retrieveNext(&label)).get().isNull()) {
+			QWidget* widget = new DebugImageView(file);
 			m_imageWidgetCleanup.add(widget);
-			m_ptrTabbedDebugImages->addTab(widget, item.label());
-			item.image() = QImage(); // Save memory.
+			m_ptrTabbedDebugImages->addTab(widget, label);
 		}
 		m_pImageFrameLayout->addWidget(m_ptrTabbedDebugImages.get());
 	}
@@ -746,19 +756,17 @@ MainWindow::contextMenuRequested(
 		goToPage(page_info.id());
 	}
 	
-	if (getCurrentView() != PageSequence::IMAGE_VIEW) {
-		// We don't support adding / removing of pages - only of images.
-		return;
-	}
-	
 	QMenu menu;
+
 	QAction* ins_before = menu.addAction(
 		QIcon(":/icons/insert-before-16.png"), tr("Insert before ...")
 	);
 	QAction* ins_after = menu.addAction(
 		QIcon(":/icons/insert-after-16.png"), tr("Insert after ...")
 	);
+
 	menu.addSeparator();
+
 	QAction* remove = menu.addAction(
 		QIcon(":/icons/user-trash.png"), tr("Remove from project ...")
 	);
@@ -769,7 +777,7 @@ MainWindow::contextMenuRequested(
 	} else if (action == ins_after) {
 		showInsertFileDialog(AFTER, page_info.imageId());
 	} else if (action == remove) {
-		showRemoveFileDialog(page_info);
+		showRemovePagesDialog(m_ptrThumbSequence->selectedItems());
 	}
 }
 
@@ -832,6 +840,8 @@ MainWindow::filterSelectionChanged(QItemSelection const& selected)
 	m_curFilter = selected.front().top();
 	bool const now_below_select_content = isBelowSelectContent(m_curFilter);
 	
+	m_ptrStages->filterAt(m_curFilter)->selected();
+
 	if (!was_below_select_content && now_below_select_content) {
 		// IMPORTANT: this needs to go before resetting thumbnails,
 		// because it may affect them.
@@ -1069,8 +1079,7 @@ MainWindow::openProject()
 	
 	QString const project_file(
 		QFileDialog::getOpenFileName(
-			0, tr("Open Project"),
-			project_dir,
+			this, tr("Open Project"), project_dir,
 			tr("Scan Tailor Projects")+" (*.ScanTailor)"
 		)
 	);
@@ -1139,6 +1148,15 @@ MainWindow::closeProject()
 	closeProjectInteractive();
 }
 
+void
+MainWindow::openSettingsDialog()
+{
+	SettingsDialog* dialog = new SettingsDialog(this);
+	dialog->setAttribute(Qt::WA_DeleteOnClose);
+	dialog->setWindowModality(Qt::WindowModal);
+	dialog->show();
+}
+
 /**
  * Note: the removed widgets are not deleted.
  */
@@ -1479,39 +1497,20 @@ MainWindow::showInsertFileDialog(BeforeOrAfter before_or_after, ImageId const& e
 }
 
 void
-MainWindow::showRemoveFileDialog(PageInfo const& page_info)
+MainWindow::showRemovePagesDialog(std::set<PageId> const& pages)
 {
-	QString const file_path(page_info.imageId().filePath());
-	QString const fname(QFileInfo(file_path).fileName());
-	
 	std::auto_ptr<QDialog> dialog(new QDialog(this));
-	Ui::RemoveFileDialog ui;
+	Ui::RemovePagesDialog ui;
 	ui.setupUi(dialog.get());
-	
-	QString image_name(fname);
-	if (page_info.isMultiPageFile()) {
-		image_name = tr("%1 (page %2)").arg(fname).arg(page_info.imageId().page() + 1);
-	}
-	ui.text->setText(ui.text->text().arg(image_name));
-	
-	ui.deleteFromDisk->setEnabled(!page_info.isMultiPageFile());
-	
+
+	ui.text->setText(ui.text->text().arg(pages.size()));
+
 	QPushButton* remove_btn = ui.buttonBox->button(QDialogButtonBox::Ok);
 	remove_btn->setText(tr("Remove"));
-	
+
 	dialog->setWindowModality(Qt::WindowModal);
-	if (dialog->exec() != QDialog::Accepted) {
-		return;
-	}
-	
-	removeFromProject(page_info.imageId());
-	if (ui.deleteFromDisk->isChecked()) {
-		if (!QFile::remove(file_path)) {
-			QMessageBox::warning(
-				0, tr("Error"),
-				tr("Unable to delete file:\n%1").arg(file_path)
-			);
-		}
+	if (dialog->exec() == QDialog::Accepted) {
+		removeFromProject(pages);
 	}
 }
 
@@ -1519,32 +1518,20 @@ void
 MainWindow::insertImage(ImageInfo const& new_image,
 	BeforeOrAfter before_or_after, ImageId const& existing)
 {
-	assert(getCurrentView() == PageSequence::IMAGE_VIEW);
-
 	m_ptrPages->insertImage(new_image, before_or_after, existing);
 	PageInfo const page_info(
 		PageId(new_image.id(), PageId::SINGLE_PAGE), new_image.metadata(),
 		new_image.isMultiPageFile(), new_image.numSubPages()
 	);
-	m_ptrThumbSequence->insert(
-		page_info, before_or_after, PageId(existing, PageId::SINGLE_PAGE)
-	);
+	m_ptrThumbSequence->insert(page_info, before_or_after, existing);
 }
 
 void
-MainWindow::removeFromProject(ImageId image_id)
+MainWindow::removeFromProject(std::set<PageId> const& pages)
 {
-	// Note that image_id is passed by value intentionally.
-	// We need a local copy, because a reference might come
-	// directly from m_ptrPages, and so will be invalidated
-	// by m_ptrPages->removeImage().
-	
-	std::vector<PageId> pages;
-	pages.push_back(PageId(image_id, PageId::SINGLE_PAGE));
-	
-	m_ptrStages->pagesRemoved(pages);
-	m_ptrPages->removeImage(image_id);
-	m_ptrThumbSequence->remove(image_id);
+	m_ptrPages->removePages(pages);
+	m_ptrStages->pageSplitFilter()->removePages(pages);
+	m_ptrThumbSequence->removePages(pages);
 	m_ptrThumbSequence->setSelection(
 		m_ptrPages->curPage(getCurrentView()).id()
 	);
