@@ -54,6 +54,7 @@ class ContentBoxPropagator;
 class ProjectCreationContext;
 class ProjectOpeningContext;
 class CompositeCacheDrivenTask;
+class TabbedDebugImages;
 class QLineF;
 class QRectF;
 class QLayout;
@@ -131,6 +132,8 @@ private slots:
 	void projectOpened(ProjectOpeningContext* context);
 	
 	void closeProject();
+
+	void openSettingsDialog();
 private:
 	enum SavePromptResult { SAVE, DONT_SAVE, CANCEL };
 	
@@ -203,13 +206,13 @@ private:
 	
 	void showInsertFileDialog(
 		BeforeOrAfter before_or_after, ImageId const& existig);
-	
-	void showRemoveFileDialog(PageInfo const& page_info);
+
+	void showRemovePagesDialog(std::set<PageId> const& pages);
 	
 	void insertImage(ImageInfo const& new_image,
 		BeforeOrAfter before_or_after, ImageId const& existing);
-	
-	void removeFromProject(ImageId image_id);
+
+	void removeFromProject(std::set<PageId> const& pages);
 	
 	BackgroundTaskPtr createCompositeTask(
 		PageInfo const& page, int page_num, int last_filter_idx);
@@ -230,7 +233,7 @@ private:
 	QStackedLayout* m_pImageFrameLayout;
 	QStackedLayout* m_pOptionsFrameLayout;
 	QPointer<FilterOptionsWidget> m_ptrOptionsWidget;
-	std::auto_ptr<QTabWidget> m_ptrTabbedDebugImages;
+	std::auto_ptr<TabbedDebugImages> m_ptrTabbedDebugImages;
 	std::auto_ptr<ContentBoxPropagator> m_ptrContentBoxPropagator;
 	std::auto_ptr<QWidget> m_ptrBatchProcessingWidget;
 	std::auto_ptr<ProcessingIndicationWidget> m_ptrProcessingIndicationWidget;
