@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,35 +19,28 @@
 #ifndef PAGE_SELECTION_ACCESSOR_H_
 #define PAGE_SELECTION_ACCESSOR_H_
 
-#include "PageSelectionProvider.h"
 #include "PageId.h"
 #include "PageRange.h"
-#include "IntrusivePtr.h"
-#include <QObject>
+#include <QPointer>
 #include <set>
 #include <vector>
 
-class PageSequence;
+class MainWindow;
 
-class PageSelectionAccessor: public QObject
+class PageSelectionAccessor
 {
-Q_OBJECT
-	// Member-wise copying is OK.
 public:
-    PageSelectionAccessor( const PageSelectionAccessor& src);
-
-	explicit PageSelectionAccessor(
-		IntrusivePtr<PageSelectionProvider const> const& provider);
+	PageSelectionAccessor(MainWindow* main_window);
+	
+	PageSelectionAccessor(PageSelectionAccessor const& other);
+	
+	PageSelectionAccessor& operator=(PageSelectionAccessor const& rhs);
 	
-	PageSequence allPages() const;
-
 	std::set<PageId> selectedPages() const;
 	
 	std::vector<PageRange> selectedRanges() const;
-signals:
-    void toBeRemoved(const std::set<PageId> pages);
 private:
-	IntrusivePtr<PageSelectionProvider const> m_ptrProvider;
+	QPointer<MainWindow> m_ptrMainWindow;
 };
 
 #endif
