@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "ProjectFilesDialog.h"
-#include "ProjectFilesDialog.moc"
+#include "ProjectFilesDialog.h.moc"
 #include "NonCopyable.h"
 #include "ImageMetadata.h"
 #include "ImageMetadataLoader.h"
@@ -36,7 +35,7 @@
 #include <QVectorIterator>
 #include <QMessageBox>
 #include <QTimerEvent>
-#include "settings/ini_keys.h"
+#include <QSettings>
 #include <QBrush>
 #include <QColor>
 #include <QDebug>
@@ -215,14 +214,11 @@ ProjectFilesDialog::ProjectFilesDialog(QWidget* parent)
 	m_metadataLoadFailed(false),
 	m_autoOutDir(true)
 {
-    QString filter = QSettings().value(_key_app_open_filetype_filter, _key_app_open_filetype_filter_def).toString();
-    foreach (QString ext, filter.toLower().split(' ', QString::SkipEmptyParts)) {
-     ext = ext.trimmed();
-     if (int i = ext.lastIndexOf('.')) {
-         ext = ext.right(ext.size()-i-1);
-     }
-     m_supportedExtensions.insert(ext);
-    }
+	m_supportedExtensions.insert("png");
+	m_supportedExtensions.insert("jpg");
+	m_supportedExtensions.insert("jpeg");
+	m_supportedExtensions.insert("tif");
+	m_supportedExtensions.insert("tiff");
 	
 	setupUi(this);
 	offProjectList->setModel(m_ptrOffProjectFilesSorted->model());
@@ -282,7 +278,7 @@ ProjectFilesDialog::inProjectFiles() const
 	using namespace boost::lambda;
 	
 	std::vector<ImageFileInfo> files;
-	m_ptrInProjectFiles->items(boost::lambda::bind(&pushFileInfo<Item>, boost::ref(files), _1));
+	m_ptrInProjectFiles->items(bind(&pushFileInfo<Item>, ref(files), _1));
 	
 	std::sort(files.begin(), files.end(), imageFileInfoLess);
 	
@@ -321,21 +317,26 @@ ProjectFilesDialog::inpDirBrowse()
 	
 	QString initial_dir(inpDirLine->text());
 	if (initial_dir.isEmpty() || !QDir(initial_dir).exists()) {
-        initial_dir = settings.value(_key_project_last_input_dir).toString();
+		initial_dir = settings.value("lastInputDir").toString();
 	}
 	if (initial_dir.isEmpty() || !QDir(initial_dir).exists()) {
 		initial_dir = QDir::home().absolutePath();
-    }
+	} else {
+		QDir dir(initial_dir);
+		if (dir.cdUp()) {
+			initial_dir = dir.absolutePath();
+		}
+	}
 	
 	QString const dir(
 		QFileDialog::getExistingDirectory(
-            this, tr("Input Directory"), initial_dir
+			this, tr("Input Directory"), initial_dir
 		)
 	);
 	
 	if (!dir.isEmpty()) {
 		setInputDir(dir);
-        settings.setValue(_key_project_last_input_dir, dir);
+		settings.setValue("lastInputDir", dir);
 	}
 }
 
@@ -406,7 +407,7 @@ ProjectFilesDialog::setInputDir(QString const& dir, bool const auto_add_files)
 	
 	inpDirLine->setText(QDir::toNativeSeparators(dir));
 	if (m_autoOutDir) {
-		setOutputDir(QDir::cleanPath(QDir(dir).filePath("out")));
+		outDirLine->setText(QDir::cleanPath(dir + QDir::separator() + "out"));
 	}
 	
 	QFileInfoList files(QDir(dir).entryInfoList(QDir::Files));
@@ -417,10 +418,8 @@ ProjectFilesDialog::setInputDir(QString const& dir, bool const auto_add_files)
 		
 		std::vector<QFileInfo> new_files(files.begin(), files.end());
 		std::vector<QFileInfo> existing_files;
-		void (std::vector<QFileInfo>::*push_back) (const QFileInfo&) =
-			&std::vector<QFileInfo>::push_back;
 		m_ptrInProjectFiles->files(
-			boost::lambda::bind(push_back, var(existing_files), _1)
+			bind(&std::vector<QFileInfo>::push_back, var(existing_files), _1)
 		);
 		std::sort(new_files.begin(), new_files.end(), FileInfoLess());
 		std::sort(existing_files.begin(), existing_files.end(), FileInfoLess());
@@ -437,9 +436,9 @@ ProjectFilesDialog::setInputDir(QString const& dir, bool const auto_add_files)
 	ItemList items;
 	std::for_each(
 		files.begin(), files.end(),
-		boost::lambda::bind(
+		bind(
 			&pushItemWithFlags<Item, ItemList>,
-            _1, boost::ref(items), boost::cref(m_supportedExtensions)
+			_1, ref(items), cref(m_supportedExtensions)
 		)
 	);
 	
@@ -454,7 +453,7 @@ ProjectFilesDialog::setInputDir(QString const& dir, bool const auto_add_files)
 void
 ProjectFilesDialog::setOutputDir(QString const& dir)
 {
-	outDirLine->setText(QDir::toNativeSeparators(dir));
+	outDirLine->setText(dir);
 }
 
 void
@@ -471,8 +470,7 @@ ProjectFilesDialog::addToProject()
 	typedef std::vector<Item> ItemList;
 	ItemList items;
 	
-	void (ItemList::*push_back) (const Item&) = &ItemList::push_back;
-	m_ptrOffProjectFiles->items(selection, boost::lambda::bind(push_back, var(items), _1));
+	m_ptrOffProjectFiles->items(selection, bind(&ItemList::push_back, var(items), _1));
 	
 	m_ptrInProjectFiles->append(items.begin(), items.end());
 	m_ptrOffProjectFiles->remove(selection);
@@ -509,9 +507,9 @@ ProjectFilesDialog::removeFromProject()
 	ItemList items;
 	
 	m_ptrInProjectFiles->items(
-		selection, boost::lambda::bind(
+		selection, bind(
 			&pushItemIfSameDir<Item, ItemList>,
-            boost::ref(items), _1, boost::cref(input_dir)
+			ref(items), _1, cref(input_dir)
 		)
 	);
 	
@@ -553,7 +551,7 @@ ProjectFilesDialog::onOK()
 		if (!m_autoOutDir) {
 			create = QMessageBox::question(
 				this, tr("Create Directory?"),
-                tr("Output directory doesn't exist. Create it?"),
+				tr("Output directory doesn't exist.  Create it?"),
 				QMessageBox::Yes|QMessageBox::No
 			) == QMessageBox::Yes;
 			if (!create) {
@@ -578,7 +576,7 @@ ProjectFilesDialog::onOK()
 		return;
 	}
 	
-    startLoadingMetadata();
+	startLoadingMetadata();
 }
 
 void
@@ -692,20 +690,15 @@ ProjectFilesDialog::FileList::remove(QItemSelection const& selection)
 	std::transform(
 		selection.begin(), selection.end(),
 		std::back_inserter(sorted_ranges),
-		boost::lambda::bind(
+		bind(
 			constructor<Range>(),
-			boost::lambda::bind(&QItemSelectionRange::top, _1),
-			boost::lambda::bind(&QItemSelectionRange::bottom, _1)
+			bind(&QItemSelectionRange::top, _1),
+			bind(&QItemSelectionRange::bottom, _1)
 		)
 	);
-	
-	// This hack is required to make it build with boost 1.44.
-	typedef int const Range::* IntMemPtr;
-	
 	std::sort(
 		sorted_ranges.begin(), sorted_ranges.end(),
-		boost::lambda::bind((IntMemPtr)&Range::first, _1) <
-			boost::lambda::bind((IntMemPtr)&Range::first, _2)
+		bind(&Range::first, _1) < bind(&Range::first, _2)
 	);
 	
 	QVectorIterator<Range> it(sorted_ranges);
@@ -767,7 +760,7 @@ ProjectFilesDialog::FileList::prepareForLoadingFiles()
 	
 	std::sort(
 		item_indexes.begin(), item_indexes.end(),
-		boost::lambda::bind(
+		bind(
 			&ItemVisualOrdering::operator(), ItemVisualOrdering(),
 			var(m_items)[_1], var(m_items)[_2]
 		)
@@ -789,11 +782,10 @@ ProjectFilesDialog::FileList::loadNextFile()
 	Item& item = m_items[item_idx];
 	std::vector<ImageMetadata> per_page_metadata;
 	QString const file_path(item.fileInfo().absoluteFilePath());
-	void (std::vector<ImageMetadata>::*push_back) (const ImageMetadata&) =
-		&std::vector<ImageMetadata>::push_back;
 	ImageMetadataLoader::Status const st = ImageMetadataLoader::load(
-		file_path, boost::lambda::bind(
-			push_back, var(per_page_metadata), _1
+		file_path, bind(
+			&std::vector<ImageMetadata>::push_back,
+			var(per_page_metadata), _1
 		)
 	);
 	
