@@ -16,21 +16,20 @@
 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "TabbedDebugImages.h"
-#include "TabbedDebugImages.moc"
+#include "TabbedDebugImages.h.moc"
 #include "DebugImageView.h"
 
 TabbedDebugImages::TabbedDebugImages(QWidget* parent)
 :	QTabWidget(parent)
 {
 	setDocumentMode(true);
-	connect(this, SIGNAL(currentChanged(int)), SLOT(currentTabChanged(int)));
+	connect(this, SIGNAL(currentChanged(QWidget*)), SLOT(currentTabChanged(QWidget*)));
 }
 
 void
-TabbedDebugImages::currentTabChanged(int const idx)
+TabbedDebugImages::currentTabChanged(QWidget* widget)
 {
-	if (DebugImageView* div = dynamic_cast<DebugImageView*>(widget(idx))) {
+	if (DebugImageView* div = dynamic_cast<DebugImageView*>(widget)) {
 		div->unlink();
 		m_liveViews.push_back(*div);
 		removeExcessLiveViews();
