@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,9 +29,9 @@ class QSizeF;
 class ThumbnailCollector : public AbstractFilterDataCollector
 {
 public:
-	virtual void processThumbnail(std::unique_ptr<QGraphicsItem>) = 0;
+	virtual void processThumbnail(std::auto_ptr<QGraphicsItem>) = 0;
 	
-	virtual IntrusivePtr<ThumbnailPixmapCache> thumbnailCache() = 0;
+	virtual ThumbnailPixmapCache& thumbnailCache() = 0;
 	
 	virtual QSizeF maxLogicalThumbSize() const = 0;
 };
