@@ -18,6 +18,7 @@
 
 #include "ImageView.h.moc"
 #include "ImageTransformation.h"
+#include "ImagePresentation.h"
 #include "imageproc/Constants.h"
 #include <QRect>
 #include <QSizeF>
@@ -25,7 +26,10 @@
 #include <QMouseEvent>
 #include <QVector>
 #include <QLineF>
+#include <QScrollBar>
+#include <QStyle>
 #include <Qt>
+#include <boost/bind.hpp>
 #include <math.h>
 
 namespace deskew
@@ -40,14 +44,42 @@ int const ImageView::m_cellSize = 20;
 ImageView::ImageView(
 	QImage const& image, QImage const& downscaled_image,
 	ImageTransformation const& xform)
-:	ImageViewBase(image, downscaled_image, xform),
-	m_imgRotationHandle(":/icons/aqua-sphere.png"),
-	m_mouseVertOffset(0.0),
-	m_state(DEFAULT_STATE)
+:	ImageViewBase(
+		image, downscaled_image,
+		ImagePresentation(xform.transform(), xform.resultingCropArea())
+	),
+	m_handlePixmap(":/icons/aqua-sphere.png"),
+	m_dragHandler(*this),
+	m_zoomHandler(*this),
+	m_xform(xform)
 {
-	m_dragHandleStatusTip = tr("Drag this handle to rotate the image.");
-	
 	setMouseTracking(true);
+
+	QString const tip(tr("Drag this handle to rotate the image."));
+	double const hit_radius = std::max<double>(0.5 * m_handlePixmap.width(), 15.0);
+	for (int i = 0; i < 2; ++i) {
+		m_handles[i].setHitRadius(hit_radius);
+		m_handles[i].setPositionCallback(
+			boost::bind(&ImageView::handlePosition, this, i)
+		);
+		m_handles[i].setMoveRequestCallback(
+			boost::bind(&ImageView::handleMoveRequest, this, i, _1)
+		);
+		m_handles[i].setDragFinishedCallback(
+			boost::bind(&ImageView::dragFinished, this)
+		);
+
+		m_handleInteractors[i].setProximityStatusTip(tip);
+		m_handleInteractors[i].setObject(&m_handles[i]);
+
+		makeLastFollower(m_handleInteractors[i]);
+	}
+
+	m_zoomHandler.setFocus(ZoomHandler::CENTER);
+
+	rootInteractionHandler().makeLastFollower(*this);
+	rootInteractionHandler().makeLastFollower(m_dragHandler);
+	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 ImageView::~ImageView()
@@ -57,25 +89,24 @@ ImageView::~ImageView()
 void
 ImageView::manualDeskewAngleSetExternally(double const degrees)
 {
-	if (imageToVirt().postRotation() == degrees) {
+	if (m_xform.postRotation() == degrees) {
 		return;
 	}
 	
-	ImageTransformation new_xform(imageToVirt());
-	new_xform.setPostRotation(degrees);
-	updateTransform(new_xform);
+	m_xform.setPostRotation(degrees);
+	updateTransform(ImagePresentation(m_xform.transform(), m_xform.resultingCropArea()));
 }
 
 void
-ImageView::paintOverImage(QPainter& painter)
+ImageView::onPaint(QPainter& painter, InteractionState const& interaction)
 {
 	painter.setWorldMatrixEnabled(false);
 	painter.setRenderHints(QPainter::Antialiasing, false);
-	
-	int const w = width();
-	int const h = height();
+
+	double const w = viewportRect().width();
+	double const h = viewportRect().height();
 	QPointF const center(getImageRotationOrigin());
-	
+
 	// Draw the semi-transparent grid.
 	QPen pen(QColor(0, 0, 255, 90));
 	pen.setCosmetic(true);
@@ -95,7 +126,7 @@ ImageView::paintOverImage(QPainter& painter)
 		lines.push_back(QLineF(x, 0.5, x, h - 0.5));
 	}
 	painter.drawLines(lines);
-	
+
 	// Draw the horizontal and vertical line crossing at the center.
 	pen.setColor(QColor(0, 0, 255));
 	painter.setPen(pen);
@@ -108,11 +139,11 @@ ImageView::paintOverImage(QPainter& painter)
 		QPointF(center.x(), 0.5),
 		QPointF(center.x(), h - 0.5)
 	);
-	
+
 	// Draw the rotation arcs.
 	// Those will look like this (  )
 	QRectF const arc_square(getRotationArcSquare());
-	
+
 	painter.setRenderHints(QPainter::Antialiasing, true);
 	pen.setWidthF(1.5);
 	painter.setPen(pen);
@@ -127,115 +158,53 @@ ImageView::paintOverImage(QPainter& painter)
 		qRound(16 * (180 - m_maxRotationDeg)),
 		qRound(16 * 2 * m_maxRotationDeg)
 	);
-	
-	// Draw rotation handles.
-	std::pair<QPointF, QPointF> const handles(getRotationHandles(arc_square));
-	QRectF handle_rect(m_imgRotationHandle.rect());
-	handle_rect.moveCenter(handles.first);
-	m_leftRotationHandle = handle_rect;
-	painter.drawPixmap(handle_rect.topLeft(), m_imgRotationHandle);
-	handle_rect.moveCenter(handles.second);
-	m_rightRotationHandle = handle_rect;
-	painter.drawPixmap(handle_rect.topLeft(), m_imgRotationHandle);
+
+	std::pair<QPointF, QPointF> const handles(
+		getRotationHandles(arc_square)
+	);
+
+	QRectF rect(m_handlePixmap.rect());
+	rect.moveCenter(handles.first);
+	painter.drawPixmap(rect.topLeft(), m_handlePixmap);
+	rect.moveCenter(handles.second);
+	painter.drawPixmap(rect.topLeft(), m_handlePixmap);
 }
 
-void
-ImageView::wheelEvent(QWheelEvent* const event)
+QPointF
+ImageView::handlePosition(int idx) const
 {
-	handleZooming(event, ZOOM_FOCUS_CENTER);
-}
-
-void
-ImageView::mousePressEvent(QMouseEvent* const event)
-{
-	if (m_state == DEFAULT_STATE && !isDraggingInProgress()) {
-		if (m_leftRotationHandle.contains(event->pos())) {
-			std::pair<QPointF, QPointF> const handles(
-				getRotationHandles(getRotationArcSquare())
-			);
-			m_mouseVertOffset = event->y() - handles.first.y();
-			m_state = DRAGGING_LEFT_HANDLE;
-			ensureCursorShape(Qt::ClosedHandCursor);
-		} else if (m_rightRotationHandle.contains(event->pos())) {
-			std::pair<QPointF, QPointF> const handles(
-				getRotationHandles(getRotationArcSquare())
-			);
-			m_mouseVertOffset = event->y() - handles.second.y();
-			m_state = DRAGGING_RIGHT_HANDLE;
-			ensureCursorShape(Qt::ClosedHandCursor);
-		}
-	}
-	
-	if (m_state == DEFAULT_STATE) {
-		handleImageDragging(event);
-	}
-}
-
-void
-ImageView::mouseReleaseEvent(QMouseEvent* const event)
-{
-	if (isDraggingInProgress()) {
-		handleImageDragging(event);
-		return;
-	}
-	
-	if (event->button() == Qt::LeftButton && m_state != DEFAULT_STATE) {
-		m_state = DEFAULT_STATE;
-		emit manualDeskewAngleSet(imageToVirt().postRotation());
-	}
-}
-
-void
-ImageView::mouseMoveEvent(QMouseEvent* const event)
-{
-	if (isDraggingInProgress()) {
-		handleImageDragging(event);
-		return;
-	}
-	
-	if (m_state == DEFAULT_STATE) {
-		if (m_leftRotationHandle.contains(event->pos()) ||
-				m_rightRotationHandle.contains(event->pos())) {
-			ensureStatusTip(m_dragHandleStatusTip);
-			if (event->buttons() & Qt::LeftButton) {
-				ensureCursorShape(Qt::ClosedHandCursor);
-			} else {
-				ensureCursorShape(Qt::OpenHandCursor);
-			}
-		} else {
-			ensureStatusTip(defaultStatusTip());
-			ensureCursorShape(Qt::ArrowCursor);
-		}
+	std::pair<QPointF, QPointF> const handles(getRotationHandles(getRotationArcSquare()));
+	if (idx == 0) {
+		return handles.first;
 	} else {
-		QRectF const arc_square(getRotationArcSquare());
-		double const arc_radius = 0.5 * arc_square.width();
-		double const abs_y = event->y() - m_mouseVertOffset;
-		double rel_y = abs_y - arc_square.center().y();
-		rel_y = qBound(-arc_radius, rel_y, arc_radius);
-		
-		double angle_rad = asin(rel_y / arc_radius);
-		if (m_state == DRAGGING_LEFT_HANDLE) {
-			angle_rad = -angle_rad;
-		}
-		double angle_deg = angle_rad * imageproc::constants::RAD2DEG;
-		angle_deg = qBound(-m_maxRotationDeg, angle_deg, m_maxRotationDeg);
-		
-		ImageTransformation new_xform(imageToVirt());
-		new_xform.setPostRotation(angle_deg);
-		updateTransformPreservingScale(new_xform);
+		return handles.second;
 	}
 }
 
 void
-ImageView::hideEvent(QHideEvent* const event)
+ImageView::handleMoveRequest(int idx, QPointF const& pos)
 {
-	ImageViewBase::hideEvent(event);
-	State const old_state = m_state;
-	m_state = DEFAULT_STATE;
-	ensureCursorShape(Qt::ArrowCursor);
-	if (old_state != DEFAULT_STATE) {
-		emit manualDeskewAngleSet(imageToVirt().postRotation());
+	QRectF const arc_square(getRotationArcSquare());
+	double const arc_radius = 0.5 * arc_square.width();
+	double const abs_y = pos.y();
+	double rel_y = abs_y - arc_square.center().y();
+	rel_y = qBound(-arc_radius, rel_y, arc_radius);
+
+	double angle_rad = asin(rel_y / arc_radius);
+	if (idx == 0) {
+		angle_rad = -angle_rad;
 	}
+	double angle_deg = angle_rad * imageproc::constants::RAD2DEG;
+	angle_deg = qBound(-m_maxRotationDeg, angle_deg, m_maxRotationDeg);
+
+	m_xform.setPostRotation(angle_deg);
+	updateTransformPreservingScale(ImagePresentation(m_xform.transform(), m_xform.resultingCropArea()));
+}
+
+void
+ImageView::dragFinished()
+{
+	emit manualDeskewAngleSet(m_xform.postRotation());
 }
 
 /**
@@ -245,9 +214,10 @@ ImageView::hideEvent(QHideEvent* const event)
 QPointF
 ImageView::getImageRotationOrigin() const
 {
+	QRectF const viewport_rect(viewportRect());
 	return QPointF(
-		floor(0.5 * width()) + 0.5,
-		floor(0.5 * height()) + 0.5
+		floor(0.5 * viewport_rect.width()) + 0.5,
+		floor(0.5 * viewport_rect.height()) + 0.5
 	);
 }
 
@@ -257,27 +227,29 @@ ImageView::getImageRotationOrigin() const
 QRectF
 ImageView::getRotationArcSquare() const
 {
-	double const h_margin = 0.5 * m_imgRotationHandle.width();
-	double const v_margin = 0.5 * m_imgRotationHandle.height();
-	
-	QRectF reduced_screen_rect(rect());
+	double const h_margin = 0.5 * m_handlePixmap.width()
+		+ verticalScrollBar()->style()->pixelMetric(QStyle::PM_ScrollBarExtent, 0, verticalScrollBar());
+	double const v_margin = 0.5 * m_handlePixmap.height()
+		+ horizontalScrollBar()->style()->pixelMetric(QStyle::PM_ScrollBarExtent, 0, horizontalScrollBar());
+
+	QRectF reduced_screen_rect(viewportRect());
 	reduced_screen_rect.adjust(h_margin, v_margin, -h_margin, -v_margin);
-	
+
 	QSizeF arc_size(1.0, m_maxRotationSin);
 	arc_size.scale(reduced_screen_rect.size(), Qt::KeepAspectRatio);
 	arc_size.setHeight(arc_size.width());
-	
+
 	QRectF arc_square(QPointF(0, 0), arc_size);
 	arc_square.moveCenter(reduced_screen_rect.center());
-	
+
 	return arc_square;
 }
 
 std::pair<QPointF, QPointF>
 ImageView::getRotationHandles(QRectF const& arc_square) const
 {
-	double const rot_sin = imageToVirt().postRotationSin();
-	double const rot_cos = imageToVirt().postRotationCos();
+	double const rot_sin = m_xform.postRotationSin();
+	double const rot_cos = m_xform.postRotationCos();
 	double const arc_radius = 0.5 * arc_square.width();
 	QPointF const arc_center(arc_square.center());
 	QPointF left_handle(-rot_cos * arc_radius, -rot_sin * arc_radius);
