@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,9 +26,6 @@
 #include <QMutex>
 #include <memory>
 #include <map>
-#include <set>
-
-class AbstractRelinker;
 
 namespace deskew
 {
@@ -37,49 +34,22 @@ class Settings : public RefCountable
 {
 	DECLARE_NON_COPYABLE(Settings)
 public:
-        enum UpdateOpt
-    {
-                 UpdateModeAndAngle,
-                 UpdateOrientationFix,
-                 UpdateAll
-};
-
-
 	Settings();
 	
 	virtual ~Settings();
 	
 	void clear();
-
-    void performRelinking(AbstractRelinker const& relinker);
-
-    void updateDeviation();
 	
 	void setPageParams(PageId const& page_id, Params const& params);
 	
 	void clearPageParams(PageId const& page_id);
 	
-	std::unique_ptr<Params> getPageParams(PageId const& page_id) const;
-	
-    void applyParams(std::set<PageId> const& pages, const Params &cur_params, UpdateOpt opt = UpdateModeAndAngle);
-
-	double maxDeviation() const { return m_maxDeviation; }
-	void setMaxDeviation(double md) { m_maxDeviation = md; }
-
-    double avg() const { return m_avg; }
-    void setAvg(double a) { m_avg = a; }
-
-    double std() const { return m_sigma; }
-    void setStd(double s) { m_sigma = s; }
-
+	std::auto_ptr<Params> getPageParams(PageId const& page_id) const;
 private:
 	typedef std::map<PageId, Params> PerPageParams;
 	
 	mutable QMutex m_mutex;
-    PerPageParams m_perPageParams;
-    double m_avg;
-    double m_sigma;
-	double m_maxDeviation;
+	PerPageParams m_perPageParams;
 };
 
 } // namespace deskew
