@@ -18,6 +18,7 @@
 
 #include "ImageView.h.moc"
 #include "ImageTransformation.h"
+#include "ImagePresentation.h"
 
 namespace fix_orientation
 {
@@ -25,8 +26,16 @@ namespace fix_orientation
 ImageView::ImageView(
 	QImage const& image, QImage const& downscaled_image,
 	ImageTransformation const& xform)
-:	ImageViewBase(image, downscaled_image, xform)
+:	ImageViewBase(
+		image, downscaled_image,
+		ImagePresentation(xform.transform(), xform.resultingCropArea())
+	),
+	m_dragHandler(*this),
+	m_zoomHandler(*this),
+	m_xform(xform)
 {
+	rootInteractionHandler().makeLastFollower(m_dragHandler);
+	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 ImageView::~ImageView()
@@ -36,37 +45,14 @@ ImageView::~ImageView()
 void
 ImageView::setPreRotation(OrthogonalRotation const rotation)
 {
-	if (imageToVirt().preRotation() == rotation) {
+	if (m_xform.preRotation() == rotation) {
 		return;
 	}
 	
-	ImageTransformation new_xform(imageToVirt());
-	new_xform.setPreRotation(rotation);
-	updateTransform(new_xform); // should call update()
-}
+	m_xform.setPreRotation(rotation);
 
-void
-ImageView::wheelEvent(QWheelEvent* const event)
-{
-	handleZooming(event);
-}
-
-void
-ImageView::mousePressEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-ImageView::mouseReleaseEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-ImageView::mouseMoveEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
+	// This should call update() by itself.
+	updateTransform(ImagePresentation(m_xform.transform(), m_xform.resultingCropArea()));
 }
 
 } // namespace fix_orientation
