@@ -17,47 +17,22 @@
 */
 
 #include "ImageView.h.moc"
-#include "ImageTransformation.h"
-#include "Dpm.h"
-#include "Dpi.h"
+#include "ImagePresentation.h"
 
 namespace output
 {
 
 ImageView::ImageView(QImage const& image, QImage const& downscaled_image)
-:	ImageViewBase(
-		image, downscaled_image,
-		ImageTransformation(image.rect(), Dpm(image))
-	)
+:	ImageViewBase(image, downscaled_image, ImagePresentation(QTransform(), QRectF(image.rect()))),
+	m_dragHandler(*this),
+	m_zoomHandler(*this)
 {
+	rootInteractionHandler().makeLastFollower(m_dragHandler);
+	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 ImageView::~ImageView()
 {
 }
 
-void
-ImageView::wheelEvent(QWheelEvent* const event)
-{
-	handleZooming(event);
-}
-
-void
-ImageView::mousePressEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-ImageView::mouseReleaseEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
-void
-ImageView::mouseMoveEvent(QMouseEvent* const event)
-{
-	handleImageDragging(event);
-}
-
 } // namespace output
