@@ -21,9 +21,11 @@
 
 #include "OutputImageParams.h"
 #include "OutputFileParams.h"
+#include "ZoneSet.h"
 
 class QDomDocument;
 class QDomElement;
+class QString;
 
 namespace output
 {
@@ -31,19 +33,27 @@ namespace output
 class OutputParams
 {
 public:
-	OutputParams(OutputImageParams const& image_params,
-		OutputFileParams const& file_params);
+	OutputParams(OutputImageParams const& output_image_params,
+		OutputFileParams const& output_file_params,
+		OutputFileParams const& automask_file_params,
+		ZoneSet const& zones);
 	
 	explicit OutputParams(QDomElement const& el);
 	
 	QDomElement toXml(QDomDocument& doc, QString const& name) const;
 	
-	OutputImageParams const& imageParams() const { return m_imageParams; }
+	OutputImageParams const& outputImageParams() const { return m_outputImageParams; }
 	
-	OutputFileParams const& fileParams() const { return m_fileParams; }
+	OutputFileParams const& outputFileParams() const { return m_outputFileParams; }
+
+	OutputFileParams const& automaskFileParams() const { return m_automaskFileParams; }
+
+	ZoneSet const& zones() const { return m_zones; }
 private:
-	OutputImageParams m_imageParams;
-	OutputFileParams m_fileParams;
+	OutputImageParams m_outputImageParams;
+	OutputFileParams m_outputFileParams;
+	OutputFileParams m_automaskFileParams;
+	ZoneSet m_zones;
 };
 
 } // namespace output
