@@ -25,6 +25,8 @@
 #include "Dpi.h"
 #include "ColorParams.h"
 #include "OutputParams.h"
+#include "ZoneSet.h"
+#include "PropertySet.h"
 #include <QMutex>
 #include <map>
 #include <memory>
@@ -65,10 +67,23 @@ public:
 	void removeOutputParams(PageId const& page_id);
 	
 	void setOutputParams(PageId const& page_id, OutputParams const& params);
+
+	ZoneSet zonesForPage(PageId const& page_id) const;
+
+	void setZones(PageId const& page_id, ZoneSet const& zones);
+
+	/**
+	 * For now, default zone properties are not persistent.
+	 * They may become persistent later though.
+	 */
+	PropertySet defaultZoneProperties() const;
+
+	void setDefaultZoneProperties(PropertySet const& props);
 private:
 	typedef std::map<PageId, Dpi> PerPageDpi;
 	typedef std::map<PageId, ColorParams> PerPageColorParams;
 	typedef std::map<PageId, OutputParams> PerPageOutputParams;
+	typedef std::map<PageId, ZoneSet> PerPageZones;
 	
 	Dpi getDpiLocked(PageId const& page_id) const;
 	
@@ -83,8 +98,10 @@ private:
 	PerPageDpi m_perPageDpi;
 	PerPageColorParams m_perPageColorParams;
 	PerPageOutputParams m_perPageOutputParams;
+	PerPageZones m_perPageZones;
 	Dpi m_defaultDpi;
 	ColorParams m_defaultColorParams;
+	PropertySet m_defaultZoneProps;
 };
 
 } // namespace output
