@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include "imageproc/ConnectivityMap.h"
 #include "imageproc/InfluenceMap.h"
 #include "imageproc/SEDM.h"
+#include <boost/foreach.hpp>
 #include <QRect>
 #include <QRectF>
 #include <QPolygonF>
@@ -61,8 +62,6 @@
 #include <stdlib.h>
 #include <limits.h>
 
-#include "CommandLine.h"
-
 namespace select_content
 {
 
@@ -111,23 +110,18 @@ struct PreferVertical
 
 QRectF
 ContentBoxFinder::findContentBox(
-	TaskStatus const& status, FilterData const& data, QRectF const& page_rect, DebugImages* dbg)
+	TaskStatus const& status, FilterData const& data, DebugImages* dbg)
 {
 	ImageTransformation xform_150dpi(data.xform());
 	xform_150dpi.preScaleToDpi(Dpi(150, 150));
-
-	if (xform_150dpi.resultingRect().toRect().isEmpty()) {
-		return QRectF();
-	}
 	
 	uint8_t const darkest_gray_level = darkestGrayLevel(data.grayImage());
-	QColor const outside_color(darkest_gray_level, darkest_gray_level, darkest_gray_level);
-
+	
 	QImage gray150(
 		transformToGray(
 			data.grayImage(), xform_150dpi.transform(),
 			xform_150dpi.resultingRect().toRect(),
-			OutsidePixels::assumeColor(outside_color)
+			QColor(darkest_gray_level, darkest_gray_level, darkest_gray_level)
 		)
 	);
 	// Note that we fill new areas that appear as a result of
@@ -137,21 +131,13 @@ ContentBoxFinder::findContentBox(
 		dbg->add(gray150, "gray150");
 	}
 	
-	BinaryImage bw150(binarizeWolf(gray150, QSize(51, 51), 50));
+	BinaryImage bw150(binarizeWolf(gray150, QSize(51, 51), 80));
 	if (dbg) {
 		dbg->add(bw150, "bw150");
 	}
 	
-	double const xscale = 150.0 / data.xform().origDpi().horizontal();
-	double const yscale = 150.0 / data.xform().origDpi().vertical();
-	QRectF page_rect150( page_rect.left()*xscale, page_rect.top()*yscale,
-		                 page_rect.right()*xscale, page_rect.bottom()*yscale );
 	PolygonRasterizer::fillExcept(
-		bw150, BLACK, page_rect150, Qt::WindingFill
-	);
-
-	PolygonRasterizer::fillExcept(
-		bw150, BLACK, xform_150dpi.resultingPreCropArea(), Qt::WindingFill
+		bw150, BLACK, xform_150dpi.resultingCropArea(), Qt::WindingFill
 	);
 	if (dbg) {
 		dbg->add(bw150, "page_mask_applied");
@@ -218,13 +204,8 @@ ContentBoxFinder::findContentBox(
 	
 	status.throwIfCancelled();
 	
-	CommandLine const& cli = CommandLine::get();
-	Despeckle::Level despeckleLevel = Despeckle::NORMAL;
-	if (cli.hasContentRect()) {
-		despeckleLevel = cli.getContentDetection();
-	}
-
-	BinaryImage despeckled(Despeckle::despeckle(content, Dpi(150, 150), despeckleLevel, status, dbg));
+	int const big_object_threshold = 25;
+	BinaryImage despeckled(despeckle(content, big_object_threshold));
 	if (dbg) {
 		dbg->add(despeckled, "despeckled");
 	}
@@ -315,11 +296,7 @@ ContentBoxFinder::findContentBox(
 		dbg->add(content_blocks, "except_bordering");
 	}
 	
-	BinaryImage text_mask(content_blocks);
-	if (cli.hasContentText()) {
-		text_mask = estimateTextMask(content, content_blocks, dbg);
-	}
-
+	BinaryImage text_mask(estimateTextMask(content, content_blocks, dbg));
 	if (dbg) {
 		QImage text_mask_visualized(content.size(), QImage::Format_ARGB32_Premultiplied);
 		text_mask_visualized.fill(0xffffffff); // Opaque white.
@@ -872,7 +849,7 @@ ContentBoxFinder::estimateTextMask(
 			}
 		}
 		
-		for (Range const range: ranges) {
+		BOOST_FOREACH (Range const range, ranges) {
 			int const first = range.first - &hist[0];
 			int const last = range.second - &hist[0];
 			if (last - first < min_text_height - 1) {
@@ -1236,6 +1213,9 @@ ContentBoxFinder::trim(
 	}
 	
 	int const content_pixels = content.countBlackPixels(removed_area);
+	if (content_pixels < 20) {
+		return new_area;
+	}
 	
 	bool const vertical_cut = (
 		new_area.top() == area.top()
@@ -1408,7 +1388,7 @@ ContentBoxFinder::filterShadows(
 	
 	status.throwIfCancelled();
 	
-	// Long white vertical lines are definitely not spaces between letters.
+	// Long white vertical lines are definately not spaces between letters.
 	BinaryImage vert_whitespace(
 		closeBrick(reduced_dithering, QSize(1, 150), BLACK)
 	);
