@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,9 +33,9 @@ namespace select_content
 class Thumbnail : public ThumbnailBase
 {
 public:
-	Thumbnail(IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
-		QSizeF const& max_size, ImageId const& image_id,
-		ImageTransformation const& xform, QRectF const& content_rect, bool deviant);
+	Thumbnail(ThumbnailPixmapCache& thumbnail_cache, QSizeF const& max_size,
+		ImageId const& image_id, ImageTransformation const& xform,
+		QRectF const& content_rect);
 	
 	virtual void paintOverImage(
 		QPainter& painter,
@@ -43,7 +43,6 @@ public:
 		QTransform const& thumb_to_display);
 private:
 	QRectF m_contentRect;
-	bool m_deviant;
 };
 
 } // namespace select_content
