@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,35 +26,19 @@
 #include <memory>
 #include <stdexcept>
 #include <algorithm>
-#include <iostream>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include "settings/ini_keys.h"
 
 namespace imageproc
 {
 
 class BinaryImage::SharedData
 {
-private:
-	/**
-	 * Resolves the ambiguity of:
-	 * \code
-	 * void SharedData::operator delete(void*, size_t);
-	 * \endcode
-	 * Which may be interpreted as both a placement and non-placement delete.
-	 */
-	struct NumWords
-	{
-		size_t numWords;
-
-		NumWords(size_t num_words) : numWords(num_words) {}
-	};
 public:
 	static SharedData* create(size_t num_words) {
-		return new(NumWords(num_words)) SharedData();
+		return new(num_words) SharedData();
 	}
 	
 	uint32_t* data() { return m_data; }
@@ -69,9 +53,9 @@ public:
 	
 	void unref() const;
 	
-	static void* operator new(size_t size, NumWords num_words);
+	static void* operator new(size_t size, size_t num_words);
 	
-	static void operator delete(void* addr, NumWords num_words);
+	static void operator delete(void* addr, size_t);
 private:
 	SharedData() : m_refCounter(1) {}
 	
@@ -560,236 +544,6 @@ BinaryImage::contentBoundingBox(BWColor const content_color) const
 	return QRect(left, top, w - right - left, bottom - top + 1);
 }
 
-
-static const int MultiplyDeBruijnBitPosition[32] =
-{
-    0, 1, 28, 2, 29, 14, 24, 3, 30, 22, 20, 15, 25, 17, 4, 8,
-    31, 27, 13, 23, 21, 19, 16, 7, 26, 12, 18, 6, 11, 5, 10, 9
-};
-
-static const int MultiplyDeBruijnBitPosition2[32] =
-{
-  0, 9, 1, 10, 13, 21, 2, 29, 11, 14, 16, 18, 22, 25, 3, 30,
-  8, 12, 20, 28, 15, 17, 24, 7, 19, 27, 23, 6, 26, 5, 4, 31
-};
-
-inline const int countConsecutiveZeroBitsTrailing(uint32_t v)
-{
-    /* aka "Count the consecutive zero bits (trailing) on the right with multiply and lookup"
-       from Bit Twiddling Hacks By Sean Eron Anderson
-       https://graphics.stanford.edu/~seander/bithacks.html#ZerosOnRightMultLookup
-     */
-    return MultiplyDeBruijnBitPosition[((uint32_t)((v & -signed(v)) * 0x077CB531U)) >> 27];
-}
-
-inline const int findPositionOfTheHighestBitSet(uint32_t v)
-{
-    /* aka "Find the log base 2 of an N-bit integer in O(lg(N)) operations with multiply and lookup"
-       from Bit Twiddling Hacks By Sean Eron Anderson
-       https://graphics.stanford.edu/~seander/bithacks.html#IntegerLogDeBruijn
-     */
-
-    v |= v >> 1; // first round down to one less than a power of 2
-    v |= v >> 2;
-    v |= v >> 4;
-    v |= v >> 8;
-    v |= v >> 16;
-
-    return MultiplyDeBruijnBitPosition2[(uint32_t)(v * 0x07C4ACDDU) >> 27];
-}
-
-void
-BinaryImage::rectangularize(BWColor content_color, std::vector<QRect>& areas, int sensitivity)
-{
-	if (isNull()) {
-		return;
-	}
-
-	int const w = m_width;
-	int const h = m_height;
-	int const wpl = m_wpl;
-	int const last_word_idx = (w - 1) >> 5;
-	int const last_word_bits = w - (last_word_idx << 5);
-	int const last_word_unused_bits = 32 - last_word_bits;
-	uint32_t const last_word_mask = ~uint32_t(0) << last_word_unused_bits;
-	uint32_t const modifier = (content_color == WHITE) ? ~uint32_t(0) : 0;
-	uint32_t const* const data = this->data();	
-
-	uint32_t const* line = data;
-	// create list of filled continuous blocks on each line
-    for (int y = 0; y < h; ++y, line += wpl) {
-        QRect area;
-        area.setTop(y);
-        area.setBottom(y);
-        bool area_found = false;
-        for (int i = 0; i <= last_word_idx; ++i) {
-            uint32_t word = line[i] ^ modifier;
-            if (i==last_word_idx) {
-                // The last (possibly incomplete) word.
-                word &= last_word_mask;
-            }
-            if (word) {
-                if (!area_found) {
-                    area.setLeft((i<<5) + 31 - findPositionOfTheHighestBitSet(~line[i]));
-                    area_found = true;
-                }
-                area.setRight(((i+1)<<5)-1);
-            } else {
-                if (area_found) {
-                    uint32_t v = line[i-1];
-                    if (v) {
-                        area.setRight(area.right() - countConsecutiveZeroBitsTrailing(~v));
-                    }
-                    areas.push_back(QRect(area));
-                    area_found = false;
-                }
-            }
-        }
-        if (area_found) {
-            uint32_t v = line[last_word_idx];
-            if (v) {
-                area.setRight(area.right() - countConsecutiveZeroBitsTrailing(~v));
-            }
-            areas.push_back(QRect(area));
-        }
-    }
-
-	// join adjacent blocks of areas
-	bool join = true;
-	int overlap = 16;
-	while (join) {
-		join = false;
-		std::vector<QRect> tmp;
-		for (QRect area: areas) {
-			// take an area and try to join with something in tmp
-			QRect enlArea(area.adjusted(-overlap,-overlap, overlap, overlap));
-			bool intersected = false;
-			std::vector<QRect> tmp2;
-			for (QRect ta: tmp) {
-				QRect enlTA(ta.adjusted(-overlap,-overlap, overlap, overlap));
-				if (enlArea.intersects(enlTA)) {
-					intersected = true;
-					join = true;
-					tmp2.push_back(area.united(ta));
-				} else {
-					tmp2.push_back(ta);
-				}
-			}
-			if (!intersected) {
-				tmp2.push_back(area);
-			}
-			tmp = tmp2;
-		}
-		areas = tmp;		
-	}
-
-    const float percent = (float)sensitivity / 100.;
-
-    if (percent < 1.) {
-        for (QRect& area: areas) {
-
-            int word_width = area.width() >> 5;
-
-            int left = area.left();
-            int left_word = left >> 5;
-            int right = area.x() + area.width();
-            int right_word = right >> 5;
-            int top = area.top();
-            int bottom = area.bottom();
-
-            uint32_t* pdata = this->data();
-
-            const int criterium = (int)(area.width() * percent);
-            const int criterium_word = (int)(word_width * percent);
-
-            // cut the dirty upper lines
-            for (int y = top; y < bottom; y++) {
-                line = pdata + m_wpl * y;
-
-                int mword = 0;
-
-                for (int k = left_word; k < right_word; k++) {
-                    if (!line[k]) {
-                        mword++; //count the totally white words
-                    }
-                }
-
-                if (mword > criterium_word) {
-                    area.setTop(y);
-                    break;
-                }
-            }
-
-            // cut the dirty bottom lines
-            for (int y = bottom; y > top; y--) {
-                line = pdata + m_wpl * y;
-
-                int mword = 0;
-
-                for (int k = left_word; k < right_word; k++) {
-                    if (!line[k]) {
-                        mword++;
-                    }
-                }
-
-                if (mword > criterium_word) {
-                    area.setBottom(y);
-                    break;
-                }
-            }
-
-            for(int x = left; x < right; x++) {
-                int mword = 0;
-
-                for (int y = top; y < bottom; y++) {
-                    if (WHITE == getPixel(x, y)) {
-                        mword++;
-                    }
-                }
-
-                if (mword > criterium) {
-                    area.setLeft(x);
-                    break;
-                }
-            }
-
-            for (int x = right; x > left; x--) {
-                int mword = 0;
-
-                for (int y = top; y < bottom; y++) {
-                    if (WHITE == getPixel(x, y)) {
-                        mword++;
-                    }
-                }
-
-                if (mword > criterium) {
-                    area.setRight(x);
-                    break;
-                }
-            }
-
-            area = area.intersected(this->rect());
-        }
-    }
-}
-
-void
-BinaryImage::setPixel(int x, int y, BWColor color)
-{
-        uint32_t* line = this->data() + m_wpl * y;
-
-        (color == WHITE) ? line[x >> 5] &= ~(0x80000000 >> (x & 31)) : line[x >> 5] |= (0x80000000 >> (x & 31));
-}
-
-BWColor
-BinaryImage::getPixel(int x, int y)
-{
-        uint32_t* line = this->data() + m_wpl * y;
-
-        return (BWColor)((line[x >> 5] >> (31 - (x & 31))) & 1);
-}
-
-
 uint32_t*
 BinaryImage::data()
 {
@@ -819,7 +573,7 @@ BinaryImage::toQImage() const
 	
 	QImage dst(m_width, m_height, QImage::Format_Mono);
 	assert(dst.bytesPerLine() % 4 == 0);
-    dst.setColorCount(2);
+	dst.setNumColors(2);
 	dst.setColor(0, 0xffffffff);
 	dst.setColor(1, 0xff000000);
 	int const dst_wpl = dst.bytesPerLine() / 4;
@@ -838,45 +592,6 @@ BinaryImage::toQImage() const
 	return dst;
 }
 
-QImage
-BinaryImage::toAlphaMask(QColor const& color) const
-{
-	if (isNull()) {
-		return QImage();
-	}
-
-	int const alpha = color.alpha();
-	int const red = (color.red() * alpha + 128) / 255;
-	int const green = (color.green() * alpha + 128) / 255;
-	int const blue = (color.blue() * alpha + 128) / 255;
-	uint32_t const colors[] = {
-		0, // replaces white
-		qRgba(red, green, blue, alpha) // replaces black
-	};
-	
-	int const width = m_width;
-	int const height = m_height;
-
-	QImage dst(width, height, QImage::Format_ARGB32_Premultiplied);
-	assert(dst.bytesPerLine() % 4 == 0);
-	
-	int const dst_stride = dst.bytesPerLine() / 4;
-	uint32_t* dst_line = (uint32_t*)dst.bits();
-	
-	uint32_t const* src_line = data();
-	int const src_stride = m_wpl;
-	
-	for (int y = 0; y < height; y++) {
-		for (int x = 0; x < width; ++x) {
-			dst_line[x] = colors[(src_line[x >> 5] >> (31 - (x & 31))) & 1];
-		}
-		src_line += src_stride;
-		dst_line += dst_stride;
-	}
-	
-	return dst;
-}
-
 void
 BinaryImage::copyIfShared()
 {
@@ -948,7 +663,7 @@ BinaryImage::fromMono(QImage const& image)
 	uint32_t* dst_line = dst.data();
 	
 	uint32_t modifier = ~uint32_t(0);
-    if (image.colorCount() >= 2) {
+	if (image.numColors() >= 2) {
 		if (qGray(image.color(0)) > qGray(image.color(1))) {
 			// if color 0 is lighter than color 1
 			modifier = ~modifier;
@@ -986,7 +701,7 @@ BinaryImage::fromMono(QImage const& image, QRect const& rect)
 	int const dst_last_word_unused_bits = (dst_wpl << 5) - width;
 	
 	uint32_t modifier = ~uint32_t(0);
-    if (image.colorCount() >= 2) {
+	if (image.numColors() >= 2) {
 		if (qGray(image.color(0)) > qGray(image.color(1))) {
 			// if color 0 is lighter than color 1
 			modifier = ~modifier;
@@ -1063,7 +778,7 @@ BinaryImage::fromIndexed8(
 	int const last_word_bits = width - (last_word_idx << 5);
 	int const last_word_unused_bits = 32 - last_word_bits;
 	
-    int const num_colors = image.colorCount();
+	int const num_colors = image.numColors();
 	assert(num_colors <= 256);
 	int color_to_gray[256];
 	int color_idx = 0;
@@ -1411,16 +1126,15 @@ void
 BinaryImage::SharedData::unref() const
 {
 	if (!m_refCounter.deref()) {
-		this->~SharedData();
-		free((void*)this);
+		delete this;
 	}
 }
 
 void*
-BinaryImage::SharedData::operator new(size_t, NumWords const num_words)
+BinaryImage::SharedData::operator new(size_t const size, size_t const num_words)
 {
 	SharedData* sd = 0;
-	void* addr = malloc(((char*)&sd->m_data[0] - (char*)sd) + num_words.numWords * 4);
+	void* addr = malloc(((char*)&sd->m_data[0] - (char*)sd) + num_words * 4);
 	if (!addr) {
 		throw std::bad_alloc();
 	}
@@ -1428,7 +1142,7 @@ BinaryImage::SharedData::operator new(size_t, NumWords const num_words)
 }
 
 void
-BinaryImage::SharedData::operator delete(void* addr, NumWords)
+BinaryImage::SharedData::operator delete(void* addr, size_t)
 {
 	free(addr);
 }
