@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,13 +18,14 @@
 
 #include "Morphology.h"
 #include "BinaryImage.h"
-#include "GrayImage.h"
 #include "RasterOp.h"
 #include "Grayscale.h"
 #include <QPoint>
 #include <QSize>
 #include <QRect>
+#include <QImage>
 #include <QDebug>
+#include <boost/foreach.hpp>
 #include <vector>
 #include <stdexcept>
 #include <algorithm>
@@ -499,13 +500,13 @@ void fillExtremumArrayRightHalf(
 
 template<typename MinOrMax>
 void spreadGrayHorizontal(
-	GrayImage& dst, GrayImage const& src,
+	QImage& dst, QImage const& src,
 	int const dy, int const dx1, int const dx2)
 {
-	int const src_stride = src.stride();
-	int const dst_stride = dst.stride();
-	uint8_t const* src_line = src.data() + dy * src_stride;
-	uint8_t* dst_line = dst.data();
+	int const src_bpl = src.bytesPerLine();
+	int const dst_bpl = dst.bytesPerLine();
+	uint8_t const* src_line = src.bits() + dy * src_bpl;
+	uint8_t* dst_line = dst.bits();
 	
 	int const dst_width = dst.width();
 	int const dst_height = dst.height();
@@ -547,15 +548,15 @@ void spreadGrayHorizontal(
 			}
 		}
 		
-		src_line += src_stride;
-		dst_line += dst_stride;
+		src_line += src_bpl;
+		dst_line += dst_bpl;
 	}
 }
 
 template<typename MinOrMax>
 void spreadGrayHorizontal(
-	GrayImage& dst, CoordinateSystem const& dst_cs,
-	GrayImage const& src, CoordinateSystem const& src_cs,
+	QImage& dst, CoordinateSystem const& dst_cs,
+	QImage const& src, CoordinateSystem const& src_cs,
 	int const dy, int const dx1, int const dx2)
 {
 	// src_point = dst_point + dst_to_src;
@@ -569,13 +570,13 @@ void spreadGrayHorizontal(
 
 template<typename MinOrMax>
 void spreadGrayVertical(
-	GrayImage& dst, GrayImage const& src,
+	QImage& dst, QImage const& src,
 	int const dx, int const dy1, int const dy2)
 {
-	int const src_stride = src.stride();
-	int const dst_stride = dst.stride();
-	uint8_t const* const src_data = src.data() + dx;
-	uint8_t* const dst_data = dst.data();
+	int const src_bpl = src.bytesPerLine();
+	int const dst_bpl = dst.bytesPerLine();
+	uint8_t const* const src_data = src.bits() + dx;
+	uint8_t* const dst_data = dst.bits();
 	
 	int const dst_width = dst.width();
 	int const dst_height = dst.height();
@@ -597,16 +598,16 @@ void spreadGrayVertical(
 				(src_segment_first + src_segment_last) >> 1;
 			
 			fillExtremumArrayLeftHalf<MinOrMax>(
-				array_center, src_data + x + src_segment_center * src_stride,
-				src_stride, src_segment_first, src_segment_center
+				array_center, src_data + x + src_segment_center * src_bpl,
+				src_bpl, src_segment_first, src_segment_center
 			);
 			
 			fillExtremumArrayRightHalf<MinOrMax>(
-				array_center, src_data + x + src_segment_center * src_stride,
-				src_stride, src_segment_center, src_segment_last
+				array_center, src_data + x + src_segment_center * src_bpl,
+				src_bpl, src_segment_center, src_segment_last
 			);
 			
-			uint8_t* dst = dst_data + x + dst_segment_first * dst_stride;
+			uint8_t* dst = dst_data + x + dst_segment_first * dst_bpl;
 			for (int y = dst_segment_first; y <= dst_segment_last; ++y) {
 				int const src_first = y + dy1;
 				int const src_last = y + dy2; // inclusive
@@ -615,7 +616,7 @@ void spreadGrayVertical(
 				uint8_t v1 = array_center[src_first - src_segment_center];
 				uint8_t v2 = array_center[src_last - src_segment_center];
 				*dst = MinOrMax::select(v1, v2);
-				dst += dst_stride;
+				dst += dst_bpl;
 			}
 		}
 	}
@@ -623,8 +624,8 @@ void spreadGrayVertical(
 
 template<typename MinOrMax>
 void spreadGrayVertical(
-	GrayImage& dst, CoordinateSystem const& dst_cs,
-	GrayImage const& src, CoordinateSystem const& src_cs,
+	QImage& dst, CoordinateSystem const& dst_cs,
+	QImage const& src, CoordinateSystem const& src_cs,
 	int const dx, int const dy1, int const dy2)
 {
 	// src_point = dst_point + dst_to_src;
@@ -636,10 +637,11 @@ void spreadGrayVertical(
 	);
 }
 
-GrayImage extendGrayImage(
-	GrayImage const& src, QRect const& dst_area, uint8_t const background)
+QImage extendGrayImage(
+	QImage const& src, QRect const& dst_area, uint8_t const background)
 {
-	GrayImage dst(dst_area.size());
+	QImage dst(dst_area.size(), QImage::Format_Indexed8);
+	dst.setColorTable(createGrayscalePalette());
 	
 	CoordinateSystem const dst_cs(dst_area.topLeft());
 	QRect const src_rect_in_dst_cs(dst_cs.fromGlobal(src.rect()));
@@ -650,14 +652,14 @@ GrayImage extendGrayImage(
 		return dst;
 	}
 	
-	uint8_t const* src_line = src.data();
-	uint8_t* dst_line = dst.data();
-	int const src_stride = src.stride();
-	int const dst_stride = dst.stride();
+	uint8_t const* src_line = src.bits();
+	uint8_t* dst_line = dst.bits();
+	int const src_bpl = src.bytesPerLine();
+	int const dst_bpl = dst.bytesPerLine();
 	
 	int y = 0;
-	for (; y < bound_src_rect_in_dst_cs.top(); ++y, dst_line += dst_stride) {
-		memset(dst_line, background, dst_stride);
+	for (; y < bound_src_rect_in_dst_cs.top(); ++y, dst_line += dst_bpl) {
+		memset(dst_line, background, dst_bpl);
 	}
 	
 	int const front_span_len = bound_src_rect_in_dst_cs.left();
@@ -669,34 +671,35 @@ GrayImage extendGrayImage(
 		bound_src_rect_in_dst_cs.topLeft() - src_rect_in_dst_cs.topLeft()
 	);
 	
-	src_line += src_offset.x() + src_offset.y() * src_stride;
+	src_line += src_offset.x() + src_offset.y() * src_bpl;
 	for (; y <= bound_src_rect_in_dst_cs.bottom(); ++y) {
 		memset(dst_line, background, front_span_len);
 		memcpy(dst_line + front_span_len, src_line, data_span_len);
 		memset(dst_line + back_span_offset, background, back_span_len);
 		
-		src_line += src_stride;
-		dst_line += dst_stride;
+		src_line += src_bpl;
+		dst_line += dst_bpl;
 	}
 	
 	int const height = dst_area.height();
-	for (; y < height; ++y, dst_line += dst_stride) {
-		memset(dst_line, background, dst_stride);
+	for (; y < height; ++y, dst_line += dst_bpl) {
+		memset(dst_line, background, dst_bpl);
 	}
 	
 	return dst;
 }
 
 template<typename MinOrMax>
-GrayImage dilateOrErodeGray(
-	GrayImage const& src, Brick const& brick,
+QImage dilateOrErodeGray(
+	QImage const& src, Brick const& brick,
 	QRect const& dst_area, unsigned char const src_surroundings)
 {
 	assert(!src.isNull());
 	assert(!brick.isEmpty());
 	assert(!dst_area.isEmpty());
 	
-	GrayImage dst(dst_area.size());
+	QImage dst(dst_area.size(), QImage::Format_Indexed8);
+	dst.setColorTable(createGrayscalePalette());
 	
 	if (!extendByBrick(src.rect(), brick).intersects(dst_area)) {
 		dst.fill(src_surroundings);
@@ -704,7 +707,7 @@ GrayImage dilateOrErodeGray(
 	}
 	CoordinateSystem const dst_cs(dst_area.topLeft());
 	
-	// Each pixel will be a minimum or maximum of a group of pixels
+	// Each pixel will be a minumum or maximum of a group of pixels
 	// in its neighborhood.  The neighborhood is defined by collect_area.
 	Brick const collect_area(brick.flipped());
 	
@@ -724,7 +727,8 @@ GrayImage dilateOrErodeGray(
 		QRect const tmp_rect(extendByBrick(dst_area, collect_area2));
 		CoordinateSystem tmp_cs(tmp_rect.topLeft());
 		
-		GrayImage tmp(tmp_rect.size());
+		QImage tmp(tmp_rect.size(), QImage::Format_Indexed8);
+		tmp.setColorTable(createGrayscalePalette());
 		
 		// First operation.  The scope is there to destroy the
 		// effective_src image when it's no longer necessary.
@@ -732,7 +736,7 @@ GrayImage dilateOrErodeGray(
 			QRect const effective_src_rect(
 				extendByBrick(tmp_rect, collect_area1)
 			);
-			GrayImage effective_src;
+			QImage effective_src;
 			CoordinateSystem effective_src_cs;
 			if (src.rect().contains(effective_src_rect)) {
 				effective_src = src;
@@ -762,7 +766,7 @@ GrayImage dilateOrErodeGray(
 		QRect const effective_src_rect(
 			extendByBrick(dst_area, collect_area)
 		);
-		GrayImage effective_src;
+		QImage effective_src;
 		CoordinateSystem effective_src_cs;
 		if (src.rect().contains(effective_src_rect)) {
 			effective_src = src;
@@ -818,8 +822,8 @@ BinaryImage dilateBrick(
 	return dst;
 }
 
-GrayImage dilateGray(
-	GrayImage const& src, Brick const& brick,
+QImage dilateGray(
+	QImage const& src, Brick const& brick,
 	QRect const& dst_area, unsigned char const src_surroundings)
 {
 	if (src.isNull()) {
@@ -831,6 +835,9 @@ GrayImage dilateGray(
 	if (dst_area.isEmpty()) {
 		throw std::invalid_argument("dilateGray: dst_area is empty");
 	}
+	if (src.format() != QImage::Format_Indexed8 || !src.isGrayscale()) {
+		throw std::invalid_argument("dilateGray: src image is not grayscale");
+	}
 	
 	return dilateOrErodeGray<Darker>(src, brick, dst_area, src_surroundings);
 }
@@ -841,8 +848,8 @@ BinaryImage dilateBrick(
 	return dilateBrick(src, brick, src.rect(), src_surroundings);
 }
 
-GrayImage dilateGray(
-	GrayImage const& src, Brick const& brick, unsigned char const src_surroundings)
+QImage dilateGray(
+	QImage const& src, Brick const& brick, unsigned char const src_surroundings)
 {
 	return dilateGray(src, brick, src.rect(), src_surroundings);
 }
@@ -870,8 +877,8 @@ BinaryImage erodeBrick(
 	return dst;
 }
 
-GrayImage erodeGray(
-	GrayImage const& src, Brick const& brick,
+QImage erodeGray(
+	QImage const& src, Brick const& brick,
 	QRect const& dst_area, unsigned char const src_surroundings)
 {
 	if (src.isNull()) {
@@ -883,6 +890,9 @@ GrayImage erodeGray(
 	if (dst_area.isEmpty()) {
 		throw std::invalid_argument("erodeGray: dst_area is empty");
 	}
+	if (src.format() != QImage::Format_Indexed8 || !src.isGrayscale()) {
+		throw std::invalid_argument("erodeGray: src image is not grayscale");
+	}
 	
 	return dilateOrErodeGray<Lighter>(src, brick, dst_area, src_surroundings);
 }
@@ -893,8 +903,8 @@ BinaryImage erodeBrick(
 	return erodeBrick(src, brick, src.rect(), src_surroundings);
 }
 
-GrayImage erodeGray(
-	GrayImage const& src, Brick const& brick, unsigned char const src_surroundings)
+QImage erodeGray(
+	QImage const& src, Brick const& brick, unsigned char const src_surroundings)
 {
 	return erodeGray(src, brick, src.rect(), src_surroundings);
 }
@@ -946,8 +956,8 @@ BinaryImage openBrick(
 	return openBrick(src, brick, src.rect(), src_surroundings);
 }
 
-GrayImage openGray(
-	GrayImage const& src, QSize const& brick,
+QImage openGray(
+	QImage const& src, QSize const& brick,
 	QRect const& dst_area, unsigned char const src_surroundings)
 {
 	if (src.isNull()) {
@@ -959,16 +969,19 @@ GrayImage openGray(
 	if (dst_area.isEmpty()) {
 		throw std::invalid_argument("openGray: dst_area is empty");
 	}
+	if (src.format() != QImage::Format_Indexed8 || !src.isGrayscale()) {
+		throw std::invalid_argument("openGray: src image is not grayscale");
+	}
 	
 	Brick const brick1(brick);
 	Brick const brick2(brick1.flipped());
 	
 	// We are going to make two operations:
 	// tmp = erodeGray(src, brick1), then dst = dilateGray(tmp, brick2)
-	QRect const tmp_rect(extendByBrick(dst_area, brick1));
+	QRect const tmp_rect(extendByBrick(dst_area, brick2));
 	CoordinateSystem tmp_cs(tmp_rect.topLeft());
 	
-	GrayImage const tmp(
+	QImage const tmp(
 		dilateOrErodeGray<Lighter>(src, brick1, tmp_rect, src_surroundings)
 	);
 	return dilateOrErodeGray<Darker>(
@@ -976,8 +989,8 @@ GrayImage openGray(
 	);
 }
 
-GrayImage openGray(
-	GrayImage const& src, QSize const& brick,
+QImage openGray(
+	QImage const& src, QSize const& brick,
 	unsigned char const src_surroundings)
 {
 	return openGray(src, brick, src.rect(), src_surroundings);
@@ -1030,8 +1043,8 @@ BinaryImage closeBrick(
 	return closeBrick(src, brick, src.rect(), src_surroundings);
 }
 
-GrayImage closeGray(
-	GrayImage const& src, QSize const& brick,
+QImage closeGray(
+	QImage const& src, QSize const& brick,
 	QRect const& dst_area, unsigned char const src_surroundings)
 {
 	if (src.isNull()) {
@@ -1043,6 +1056,9 @@ GrayImage closeGray(
 	if (dst_area.isEmpty()) {
 		throw std::invalid_argument("closeGray: dst_area is empty");
 	}
+	if (src.format() != QImage::Format_Indexed8 || !src.isGrayscale()) {
+		throw std::invalid_argument("closeGray: src image is not grayscale");
+	}
 	
 	Brick const brick1(brick);
 	Brick const brick2(brick1.flipped());
@@ -1052,7 +1068,7 @@ GrayImage closeGray(
 	QRect const tmp_rect(extendByBrick(dst_area, brick2));
 	CoordinateSystem tmp_cs(tmp_rect.topLeft());
 	
-	GrayImage const tmp(
+	QImage const tmp(
 		dilateOrErodeGray<Darker>(src, brick1, tmp_rect, src_surroundings)
 	);
 	return dilateOrErodeGray<Lighter>(
@@ -1060,8 +1076,8 @@ GrayImage closeGray(
 	);
 }
 
-GrayImage closeGray(
-	GrayImage const& src, QSize const& brick,
+QImage closeGray(
+	QImage const& src, QSize const& brick,
 	unsigned char const src_surroundings)
 {
 	return closeGray(src, brick, src.rect(), src_surroundings);
@@ -1080,7 +1096,7 @@ BinaryImage hitMissMatch(
 	
 	bool first = true;
 	
-	for (QPoint const& hit: hits) {
+	BOOST_FOREACH (QPoint const& hit, hits) {
 		QRect src_rect(rect);
 		QRect dst_rect(rect.translated(-hit));
 		adjustToFit(rect, dst_rect, src_rect);
@@ -1103,7 +1119,7 @@ BinaryImage hitMissMatch(
 		}
 	}
 	
-	for (QPoint const& miss: misses) {
+	BOOST_FOREACH (QPoint const& miss, misses) {
 		QRect src_rect(rect);
 		QRect dst_rect(rect.translated(-miss));
 		adjustToFit(rect, dst_rect, src_rect);
@@ -1245,7 +1261,7 @@ void hitMissReplaceInPlace(
 	BinaryImage const matches(hitMissMatch(img, src_surroundings, hits, misses));
 	QRect const rect(img.rect());
 		
-	for (QPoint const& offset: white_to_black) {
+	BOOST_FOREACH (QPoint const& offset, white_to_black) {
 		QRect src_rect(rect);
 		QRect dst_rect(rect.translated(offset));
 		adjustToFit(rect, dst_rect, src_rect);
@@ -1255,7 +1271,7 @@ void hitMissReplaceInPlace(
 		);
 	}
 	
-	for (QPoint const& offset: black_to_white) {
+	BOOST_FOREACH (QPoint const& offset, black_to_white) {
 		QRect src_rect(rect);
 		QRect dst_rect(rect.translated(offset));
 		adjustToFit(rect, dst_rect, src_rect);
