@@ -4,29 +4,37 @@
 <context>
     <name>CrashReportDialog</name>
     <message>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="62"/>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="70"/>
         <source>This file contains the internal state of the program when it crashed</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="68"/>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="76"/>
         <source>Dump file</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="98"/>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="104"/>
         <source>Sending ...</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="126"/>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="149"/>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="207"/>
-        <source>Sending failed</source>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="135"/>
+        <source>Unexpected response (code %1) from dispatcher</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <location filename="../crash_reporter/CrashReportDialog.cpp" line="212"/>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="160"/>
+        <source>Unexpected response from dispatcher</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="220"/>
+        <source>Unexpected response (code %1) from the receiving side</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location filename="../crash_reporter/CrashReportDialog.cpp" line="226"/>
         <source>Successfully sent</source>
         <translation type="unfinished"></translation>
     </message>
@@ -37,7 +45,7 @@
     </message>
     <message>
         <location filename="../crash_reporter/ui/CrashReportDialog.ui" line="20"/>
-        <source>Scan Tailor crashed.  That&apos;s usually caused by errors in the software.  You can help us find and fix this problem by submitting this crash report.</source>
+        <source>Scan Tailor crashed.  That&apos;s usually caused by errors in the software.  You can help us find and fix the problem by submitting this crash report.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
