@@ -1,6 +1,6 @@
 /*
 	Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+	Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,12 +22,8 @@
 #include <QVector>
 #include <QPointF>
 #include <QPolygonF>
-#ifndef Q_MOC_RUN
-#include <boost/function.hpp>
-#endif
 
 class EditableSpline;
-class QTransform;
 class QDomDocument;
 class QDomElement;
 class QString;
@@ -39,21 +35,9 @@ public:
 
 	explicit SerializableSpline(QDomElement const& el);
 
-//begin of modified by monday2000
-//Quadro_Zoner
-	SerializableSpline(QPolygonF const& polygon);
-//end of modified by monday2000	
-
 	QDomElement toXml(QDomDocument& doc, QString const& name) const;
 
-	SerializableSpline transformed(QTransform const& xform) const;
-
-	SerializableSpline transformed(
-		boost::function<QPointF(QPointF const&)> const& xform) const;
-
-    QPolygonF toPolygon() const { return QPolygonF(m_points); }
-
-    const QVector<QPointF>& points() const { return m_points; }
+	QPolygonF toPolygon() const { return QPolygonF(m_points); }
 private:
 	QVector<QPointF> m_points;
 };
